@@ -22,6 +22,7 @@
  */
 
 #include "config.h"
+#include <gst/gst.h>
 #ifdef HAVE_FFMPEG_UNINSTALLED
 #include <avcodec.h>
 #include <avformat.h>
@@ -30,18 +31,20 @@
 #include <ffmpeg/avformat.h>
 #endif
 
-#include <gst/gst.h>
-
 extern gboolean gst_ffmpegdemux_register (GstPlugin *plugin);
 extern gboolean gst_ffmpegdec_register (GstPlugin *plugin);
 extern gboolean gst_ffmpegenc_register (GstPlugin *plugin);
 extern gboolean gst_ffmpegmux_register (GstPlugin *plugin);
+extern gboolean gst_ffmpegcsp_register (GstPlugin *plugin);
 	
 extern URLProtocol gstreamer_protocol;
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
+
   avcodec_init ();
   avcodec_register_all ();
   av_register_all ();
@@ -50,6 +53,7 @@ plugin_init (GModule *module, GstPlugin *plugin)
   gst_ffmpegdec_register (plugin);
   /*gst_ffmpegdemux_register (plugin);*/
   /*gst_ffmpegmux_register (plugin);*/
+  gst_ffmpegcsp_register (plugin);
 
   /*register_protocol (&gstreamer_protocol);*/
 
@@ -57,9 +61,15 @@ plugin_init (GModule *module, GstPlugin *plugin)
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "ffmpeg",
-  plugin_init
-};
+  "All FFMPEG codecs",
+  plugin_init,
+  FFMPEG_VERSION,
+  "LGPL",
+  "(c) 2003 The FFMPEG team",
+  "FFMpeg",
+  "http://ffmpeg.sourceforge.net/"
+)
