@@ -20,13 +20,13 @@
  */
 
 #include "config.h"
+#include <gst/gst.h>
 #ifdef HAVE_FFMPEG_UNINSTALLED
 #include <avcodec.h>
 #else
 #include <ffmpeg/avcodec.h>
 #endif
 #include <string.h>
-#include <gst/gst.h>
 
 #include "gstffmpegcodecmap.h"
 
@@ -44,7 +44,7 @@
 		      "height", GST_PROPS_INT (context->height),\
 		      "framerate", GST_PROPS_FLOAT (            \
 				1.*context->frame_rate/         \
-				context->frame_rate_base),      \
+				context->frame_rate_base) ,     \
 		      ##props)					\
 	:							\
 	GST_CAPS_NEW (name,					\
@@ -52,7 +52,7 @@
 		      "width",  GST_PROPS_INT_RANGE (16, 4096),	\
 		      "height", GST_PROPS_INT_RANGE (16, 4096),	\
 		      "framerate", GST_PROPS_FLOAT_RANGE (0,    \
-						G_MAXFLOAT),    \
+						G_MAXFLOAT) ,   \
 		      ##props)
 
 /* same for audio - now with channels/sample rate
@@ -63,13 +63,11 @@
 	GST_CAPS_NEW (name,						\
 		      mimetype,						\
 		      "rate",     GST_PROPS_INT (context->sample_rate),	\
-		      "channels", GST_PROPS_INT (context->channels),	\
+		      "channels", GST_PROPS_INT (context->channels) ,	\
 		      ##props)						\
 	:								\
 	GST_CAPS_NEW (name,						\
 		      mimetype,						\
-		      "rate",     GST_PROPS_INT_RANGE (8000, 96000),	\
-		      "channels", GST_PROPS_INT_RANGE (1, 2),		\
 		      ##props)
 
 /* Convert a FFMPEG codec ID and optional AVCodecContext
@@ -233,7 +231,8 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
 
     case CODEC_ID_DVAUDIO:
         caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_dvaudio",
-                                    "audio/x-dv"
+                                    "audio/x-dv",
+				    NULL
                                    );
         break;
 
@@ -246,14 +245,22 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
     case CODEC_ID_WMAV1:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_wma1",
                                   "audio/x-wma",
-                                    "wmaversion", GST_PROPS_INT (1)
+                                    "wmaversion",  GST_PROPS_INT (1),
+                                    "flags1",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                    "flags2",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                    "block_align", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                                    "bitrate",     GST_PROPS_INT_RANGE (0, G_MAXINT)
                                  );
       break;
 
     case CODEC_ID_WMAV2:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_wma2",
                                   "audio/x-wma",
-                                    "wmaversion", GST_PROPS_INT (2)
+                                    "wmaversion",  GST_PROPS_INT (2),
+                                    "flags1",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                    "flags2",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                    "block_align", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                                    "bitrate",     GST_PROPS_INT_RANGE (0, G_MAXINT)
                                  );
       break;
 
@@ -305,7 +312,11 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
       break;
 
     case CODEC_ID_AAC:
-      /* .. */
+      caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_mpeg2aac",
+				  "audio/mpeg",
+				    "systemstream", GST_PROPS_BOOLEAN (FALSE),
+				    "mpegversion",  GST_PROPS_INT (2)
+				 );
       break;
 
     case CODEC_ID_MPEG4AAC:
@@ -393,12 +404,14 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
 
     case CODEC_ID_PCM_MULAW:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_mulawaudio",
-                                  "audio/x-mulaw");
+                                  "audio/x-mulaw",
+				  NULL);
       break;
 
     case CODEC_ID_PCM_ALAW:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_alawaudio",
-                                  "audio/x-alaw");
+                                  "audio/x-alaw",
+				  NULL);
       break;
 
     case CODEC_ID_ADPCM_IMA_QT:
@@ -433,7 +446,7 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
       /* .. */
       break;
 
-    /*case CODEC_ID_RA_144:
+    case CODEC_ID_RA_144:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_realaudio_144",
                                   "audio/x-pn-realaudio",
                                     "bitrate", GST_PROPS_INT (14400)
@@ -445,7 +458,7 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
                                   "audio/x-pn-realaudio",
                                     "bitrate", GST_PROPS_INT (28800)
                                  );
-      break;*/
+      break;
 
     default:
       /* .. */
@@ -675,6 +688,69 @@ gst_ffmpeg_codectype_to_caps (enum CodecType  codec_type,
   return caps;
 }
 
+
+/* Construct the context extradata from caps
+ * when needed.
+ */
+static void
+gst_ffmpeg_caps_to_extradata (GstCaps        *caps,
+                              AVCodecContext *context)
+{
+  const gchar *mimetype;
+
+  mimetype = gst_caps_get_mime(caps);
+
+  if (!strcmp(mimetype, "audio/x-wma")) {
+
+    if (!gst_caps_has_property (caps, "flags1")) {
+      g_warning ("Caps without flags1 property for %s", mimetype);
+      return;
+    }
+
+    if (!gst_caps_has_property (caps, "flags2")) {
+      g_warning ("Caps without flags2 property for %s", mimetype);
+      return;
+    }
+
+    if (gst_caps_has_property (caps, "wmaversion")) {
+      gint wmaversion = 0;
+      gint value;
+
+      /* 
+       * Rebuild context data from flags1 & flags2 
+       * see wmadec in ffmpeg/libavcodec/wmadec.c 
+       */
+      gst_caps_get_int (caps, "wmaversion", &wmaversion);
+      switch (wmaversion) {
+        case 1:
+          context->extradata = (guint8 *) g_malloc0 (4); 
+          gst_caps_get_int (caps, "flags1", &value);
+          ((guint8 *)context->extradata)[0] = value;
+          gst_caps_get_int (caps, "flags2", &value);
+          ((guint8 *)context->extradata)[2] = value;
+          context->extradata_size = 4;
+          break;
+        case 2:
+          context->extradata = (guint8 *) g_malloc0 (6);        
+          gst_caps_get_int (caps, "flags1", &value);
+          ((guint8 *) context->extradata)[0] = value;
+          ((guint8 *) context->extradata)[1] = value >> 8;
+          ((guint8 *) context->extradata)[2] = value >> 16;
+          ((guint8 *) context->extradata)[3] = value >> 24;
+          gst_caps_get_int (caps, "flags2", &value);
+          ((guint8 *) context->extradata)[4] = value; 
+          ((guint8 *) context->extradata)[5] = value >> 8;
+          context->extradata_size = 6;
+          break;
+        default:
+          g_warning ("Unknown wma version %d\n", wmaversion);
+          break;
+      }
+    }
+  }
+}
+
+
 /* Convert a GstCaps (audio/raw) to a FFMPEG SampleFmt
  * and other audio properties in a AVCodecContext.
  *
@@ -709,15 +785,28 @@ gst_ffmpeg_caps_to_smpfmt (GstCaps        *caps,
 
   if (gst_caps_has_property_typed (caps, "channels",
 				   GST_PROPS_INT_TYPE)) {
-    gst_caps_get_int (caps, "rate", &context->channels);
+    gst_caps_get_int (caps, "channels", &context->channels);
   }
 
   if (gst_caps_has_property_typed (caps, "rate",
 				   GST_PROPS_INT_TYPE)) {
     gst_caps_get_int (caps, "rate", &context->sample_rate);
   }
+
+  if (gst_caps_has_property_typed (caps, "block_align",
+				   GST_PROPS_INT_TYPE)) {
+    gst_caps_get_int (caps, "block_align", &context->block_align);
+  }
+
+  if (gst_caps_has_property_typed (caps, "bitrate",
+				   GST_PROPS_INT_TYPE)) {
+    gst_caps_get_int (caps, "bitrate", &context->bit_rate);
+  }
+
+  gst_ffmpeg_caps_to_extradata (caps, context);
 }
 
+
 /* Convert a GstCaps (video/raw) to a FFMPEG PixFmt
  * and other video properties in a AVCodecContext.
  *
@@ -742,8 +831,8 @@ gst_ffmpeg_caps_to_pixfmt (GstCaps        *caps,
 			           GST_PROPS_FLOAT_TYPE)) {
     gfloat fps;
     gst_caps_get_float (caps, "framerate", &fps);
-    context->frame_rate = fps * 1000;
-    context->frame_rate_base = 1000;
+    context->frame_rate = fps * DEFAULT_FRAME_RATE_BASE;
+    context->frame_rate_base = DEFAULT_FRAME_RATE_BASE;
   }
 
   if (gst_caps_has_property_typed (caps, "format",
@@ -758,9 +847,17 @@ gst_ffmpeg_caps_to_pixfmt (GstCaps        *caps,
       case GST_MAKE_FOURCC ('I','4','2','0'):
         context->pix_fmt = PIX_FMT_YUV420P;
         break;
-      case GST_MAKE_FOURCC ('Y','4','1','P'):
+      case GST_MAKE_FOURCC ('Y','4','1','B'):
         context->pix_fmt = PIX_FMT_YUV411P;
         break;
+      case GST_MAKE_FOURCC ('Y','U','V','9'):
+        context->pix_fmt = PIX_FMT_YUV410P;
+        break;
+#if 0
+      case FIXME:
+        context->pix_fmt = PIX_FMT_YUV444P;
+        break;
+#endif
       case GST_MAKE_FOURCC ('R','G','B',' '):
         if (gst_caps_has_property_typed (caps, "depth",
 					 GST_PROPS_INT_TYPE) &&
