@@ -18,6 +18,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <string.h>
 
@@ -25,17 +28,14 @@
 
 #include "flac_compat.h"
 
-extern GstPadTemplate *gst_flacenc_src_template, *gst_flacenc_sink_template;
+static GstPadTemplate *src_template, *sink_template;
 
 /* elementfactory information */
 GstElementDetails flacenc_details = {
   "FLAC encoder",
-  "Codec/Audio/Encoder",
-  "LGPL",
+  "Codec/Encoder/Audio",
   "Encodes audio with the FLAC lossless audio encoder",
-  VERSION,
   "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
 };
 
 /* FlacEnc signals and args */
@@ -59,16 +59,16 @@ enum {
   ARG_MIN_RESIDUAL_PARTITION_ORDER,
   ARG_MAX_RESIDUAL_PARTITION_ORDER,
   ARG_RICE_PARAMETER_SEARCH_DIST,
-  ARG_METADATA
 };
 
+static void             gst_flacenc_base_init           (gpointer g_class);
 static void		gst_flacenc_init		(FlacEnc *flacenc);
 static void		gst_flacenc_class_init		(FlacEncClass *klass);
 static void 		gst_flacenc_dispose 		(GObject *object);
 
 static GstPadLinkReturn
 			gst_flacenc_sinkconnect 	(GstPad *pad, GstCaps *caps);
-static void		gst_flacenc_chain		(GstPad *pad, GstBuffer *buf);
+static void		gst_flacenc_chain		(GstPad *pad, GstData *_data);
 
 static gboolean 	gst_flacenc_update_quality 	(FlacEnc *flacenc, gint quality);
 static void     	gst_flacenc_set_property        (GObject *object, guint prop_id, 
@@ -99,7 +99,7 @@ flacenc_get_type (void)
   if (!flacenc_type) {
     static const GTypeInfo flacenc_info = {
       sizeof(FlacEncClass),
-      NULL,
+      gst_flacenc_base_init,
       NULL,
       (GClassInitFunc)gst_flacenc_class_init,
       NULL,
@@ -167,6 +167,56 @@ gst_flacenc_quality_get_type (void)
   return qtype;
 }
 
+static GstCaps*
+flac_caps_factory (void)
+{
+  return
+   gst_caps_new (
+  	"flac_flac",
+  	"application/x-flac",
+	/* gst_props_new (
+ 	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL) */NULL);
+}
+
+static GstCaps*
+raw_caps_factory (void)
+{
+  return
+   gst_caps_new (
+  	"flac_raw",
+  	"audio/x-raw-int",
+	gst_props_new (
+    	    "endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
+    	    "signed", 		GST_PROPS_BOOLEAN (TRUE),
+    	    "width", 		GST_PROPS_INT (16),
+    	    "depth",    	GST_PROPS_INT (16),
+    	    "rate",     	GST_PROPS_INT_RANGE (11025, 48000),
+    	    "channels", 	GST_PROPS_INT_RANGE (1, 2),
+	    NULL));
+}
+
+static void
+gst_flacenc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *flac_caps;
+
+  raw_caps = raw_caps_factory ();
+  flac_caps = flac_caps_factory ();
+  
+  sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
+					GST_PAD_ALWAYS, 
+					raw_caps, NULL);
+  src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
+				       GST_PAD_ALWAYS, 
+				       flac_caps, NULL);
+  gst_element_class_add_pad_template (element_class, sink_template);
+  gst_element_class_add_pad_template (element_class, src_template);
+  gst_element_class_set_details (element_class, &flacenc_details);
+}
+
 static void
 gst_flacenc_class_init (FlacEncClass *klass)
 {
@@ -257,9 +307,6 @@ gst_flacenc_class_init (FlacEncClass *klass)
 		         "parameters, use best",
                        0, FLAC__MAX_RICE_PARTITION_ORDER, 
                        flacenc_params[DEFAULT_QUALITY].rice_parameter_search_dist, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METADATA,
-    g_param_spec_boxed ("metadata", "Metadata", "Metadata to add to the stream,",
-                        GST_TYPE_CAPS, G_PARAM_READWRITE));		       
 
   gstelement_class->change_state = gst_flacenc_change_state;
 }
@@ -267,12 +314,12 @@ gst_flacenc_class_init (FlacEncClass *klass)
 static void
 gst_flacenc_init (FlacEnc *flacenc)
 {
-  flacenc->sinkpad = gst_pad_new_from_template (gst_flacenc_sink_template, "sink");
+  flacenc->sinkpad = gst_pad_new_from_template (sink_template, "sink");
   gst_element_add_pad(GST_ELEMENT(flacenc),flacenc->sinkpad);
   gst_pad_set_chain_function(flacenc->sinkpad,gst_flacenc_chain);
   gst_pad_set_link_function (flacenc->sinkpad, gst_flacenc_sinkconnect);
 
-  flacenc->srcpad = gst_pad_new_from_template (gst_flacenc_src_template, "src");
+  flacenc->srcpad = gst_pad_new_from_template (src_template, "src");
   gst_element_add_pad(GST_ELEMENT(flacenc),flacenc->srcpad);
 
   GST_FLAG_SET (flacenc, GST_ELEMENT_EVENT_AWARE);
@@ -290,13 +337,20 @@ gst_flacenc_init (FlacEnc *flacenc)
   flacenc->metadata = GST_CAPS_NEW (
                   "flacenc_metadata",
                   "application/x-gst-metadata",
-		    "comment",  GST_PROPS_STRING ("Track encoded with GStreamer"),
-                   "date",     GST_PROPS_STRING (""),
-		    "tracknum", GST_PROPS_STRING (""),
-		    "title",    GST_PROPS_STRING (""),
-		    "artist",   GST_PROPS_STRING (""),
-		    "album",    GST_PROPS_STRING (""),
-		    "genre",    GST_PROPS_STRING ("")
+                    "DESCRIPTION",      GST_PROPS_STRING ("Track encoded with GStreamer"),
+                    "DATE",             GST_PROPS_STRING (""),
+                    "TRACKNUMBER",      GST_PROPS_STRING (""),
+                    "TITLE",            GST_PROPS_STRING (""),
+                    "ARTIST",           GST_PROPS_STRING (""),
+                    "ALBUM",            GST_PROPS_STRING (""),
+                    "GENRE",            GST_PROPS_STRING (""),
+                    "VERSION",          GST_PROPS_STRING (""),
+                    "COPYRIGHT",        GST_PROPS_STRING (""),
+                    "LICENSE",          GST_PROPS_STRING (""),
+                    "ORGANISATION",     GST_PROPS_STRING (""),
+                    "LOCATION",         GST_PROPS_STRING (""),
+                    "CONTACT",          GST_PROPS_STRING (""),
+                    "ISRC",             GST_PROPS_STRING ("")
                   );
 		  
   flacenc->negotiated = FALSE;
@@ -319,6 +373,7 @@ gst_flacenc_dispose (GObject *object)
 static GstPadLinkReturn
 gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
 {
+  GstPadLinkReturn ret;
   FlacEnc *flacenc;
 
   flacenc = GST_FLACENC (gst_pad_get_parent (pad));
@@ -330,6 +385,15 @@ gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
   gst_caps_get_int (caps, "depth", &flacenc->depth);
   gst_caps_get_int (caps, "rate", &flacenc->sample_rate);
   
+  caps = GST_CAPS_NEW ("flacenc_srccaps",
+                       "application/x-flac",
+                         "channels", GST_PROPS_INT (flacenc->channels),
+                         "rate", GST_PROPS_INT (flacenc->sample_rate));
+  ret = gst_pad_try_set_caps (flacenc->srcpad, caps);
+  if (ret <= 0) {
+    return ret;
+  }
+
   FLAC__seekable_stream_encoder_set_bits_per_sample (flacenc->encoder, 
 		  			    flacenc->depth);
   FLAC__seekable_stream_encoder_set_sample_rate (flacenc->encoder, 
@@ -339,7 +403,7 @@ gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
 
   flacenc->negotiated = TRUE;
 
-  return GST_PAD_LINK_OK;
+  return ret;
 }
 
 static gboolean
@@ -394,7 +458,7 @@ GstEvent *event;
   event = gst_event_new_seek ((GstSeekType)(int)(GST_FORMAT_BYTES | GST_SEEK_METHOD_SET), absolute_byte_offset); 
 
   if (event)
-    gst_pad_push (flacenc->srcpad, GST_BUFFER (event));
+    gst_pad_push (flacenc->srcpad, GST_DATA (event));
  
   return  FLAC__STREAM_ENCODER_OK;
 }
@@ -423,7 +487,7 @@ gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder,
     flacenc->first = FALSE;
   }
 
-  gst_pad_push (flacenc->srcpad, outbuf);
+  gst_pad_push (flacenc->srcpad, GST_DATA (outbuf));
 
   return FLAC__STREAM_ENCODER_OK;
 }
@@ -431,40 +495,35 @@ gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder,
 static void
 gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
 {
-  GList *props;
-  GstPropsEntry *prop;
+  guint indice;
   guint comments;
+  const gchar *meta_types[] = { "TITLE", "VERSION", "ALBUM", "TRACKNUMBER",
+                                "ARTIST", "PERFORMER", "COPYRIGHT", "LICENSE",
+                                "ORGANISATION", "DESCRIPTION", "GENRE", "DATE",
+                                "LOCATION", "CONTACT", "ISRC", NULL };
 
   g_return_if_fail (flacenc != NULL);
 
   flacenc->meta = g_malloc (sizeof (FLAC__StreamMetadata **));
 
   flacenc->meta[0] = FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
-
-  comments = 0;
-  props = gst_caps_get_props (caps)->properties;
-  while (props) {
-    prop = (GstPropsEntry*)(props->data);
-    props = g_list_next(props);
-
-    if (gst_props_entry_get_type (prop) == GST_PROPS_STRING_TYPE) {
-      const gchar *name = gst_props_entry_get_name (prop);
-      const gchar *value;
+  
+  for ( indice = 0, comments=0; meta_types[indice] != NULL; indice++) {
+    if (gst_caps_has_property(caps, meta_types[indice])) {
+      const gchar *entry;
       gchar *data;
       FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
+      
+      gst_caps_get_string(caps, (gchar *)meta_types[indice], &entry);
 
-      gst_props_entry_get_string (prop, &value);
-
-      if (!value || strlen (value) == 0)
-        continue;
-
-      data = g_strdup_printf("%s=%s", name, value);
-
-      commment_entry.length = GUINT32_TO_LE (strlen(data));
-      commment_entry.entry  = g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
+      if (!strcmp (entry, ""))
+	continue;
+   
+      data = g_strdup_printf("%s=%s", meta_types[indice], entry);
+      commment_entry.length = GUINT32_TO_LE (strlen(entry) + strlen(meta_types[indice]) + 1);
+      commment_entry.entry  =  g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
 
       FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], comments++, commment_entry, TRUE);
-      g_free (data);
     }
   }
 
@@ -474,8 +533,9 @@ gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
 
 
 static void
-gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
+gst_flacenc_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   FlacEnc *flacenc;
   FLAC__int32 *data;
   gulong insize;
@@ -617,9 +677,6 @@ gst_flacenc_set_property (GObject *object, guint prop_id,
     case ARG_RICE_PARAMETER_SEARCH_DIST:
       FLAC__seekable_stream_encoder_set_rice_parameter_search_dist (this->encoder, 
 		                                 g_value_get_uint (value));
-      break; 
-    case ARG_METADATA:
-      this->metadata = g_value_get_boxed (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -687,9 +744,6 @@ gst_flacenc_get_property (GObject *object, guint prop_id,
       g_value_set_uint (value, 
 	      FLAC__seekable_stream_encoder_get_rice_parameter_search_dist (this->encoder));
       break;
-    case ARG_METADATA:
-      g_value_set_static_boxed (value, this->metadata);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -722,8 +776,11 @@ gst_flacenc_change_state (GstElement *element)
       flacenc->first_buf = NULL;
       g_free (flacenc->data);
       flacenc->data = NULL;
-      FLAC__metadata_object_delete (flacenc->meta[0]);
-      g_free (flacenc->meta);
+      if (flacenc->meta) {
+	FLAC__metadata_object_delete (flacenc->meta[0]);
+	g_free (flacenc->meta);
+	flacenc->meta = NULL;
+      }
       break;
     case GST_STATE_READY_TO_NULL:
     default:
