@@ -16,21 +16,22 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include "gstpngenc.h"
+#include <gst/video/video.h>
 
 #define MAX_HEIGHT		4096
 
 
 GstElementDetails gst_pngenc_details = {
-  "",
-  "Filter/Video/Effect",
-  "LGPL",
-  "Encode a video frame to a .png image"
-  VERSION,
+  "PNG encoder",
+  "Codec/Encoder/Image",
+  "Encode a video frame to a .png image",
   "Jeremy SIMON <jsimon13@yahoo.fr>",
-  "(C) 2000 Donald Graft",
 };
 
 
@@ -46,10 +47,11 @@ enum
   ARG_0
 };
 
+static void     gst_pngenc_base_init    (gpointer g_class);
 static void	gst_pngenc_class_init	(GstPngEncClass *klass);
 static void	gst_pngenc_init		(GstPngEnc *pngenc);
 
-static void	gst_pngenc_chain	(GstPad *pad, GstBuffer *buf);
+static void	gst_pngenc_chain	(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 
@@ -71,7 +73,8 @@ GType gst_pngenc_get_type (void)
 
   if (!pngenc_type) {
     static const GTypeInfo pngenc_info = {
-      sizeof (GstPngEncClass), NULL,
+      sizeof (GstPngEncClass),
+      gst_pngenc_base_init,
       NULL,
       (GClassInitFunc) gst_pngenc_class_init,
       NULL,
@@ -87,6 +90,50 @@ GType gst_pngenc_get_type (void)
   return pngenc_type;
 }
 
+static GstCaps*
+png_caps_factory (void)
+{
+  return gst_caps_new ( "png_png",
+			"video/x-png",
+			gst_props_new (
+			  "width",     GST_PROPS_INT_RANGE (16, 4096),
+			  "height",    GST_PROPS_INT_RANGE (16, 4096),
+			  "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
+			  NULL));
+}
+
+
+static GstCaps*
+raw_caps_factory (void)
+{ 
+  return gst_caps_new ( "png_raw", 
+  			"video/x-raw-rgb",
+			 GST_VIDEO_RGB_PAD_TEMPLATE_PROPS_24
+	              );
+}
+
+static void
+gst_pngenc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *png_caps;
+  
+  raw_caps = raw_caps_factory ();
+  png_caps = png_caps_factory ();
+
+  pngenc_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK,
+					       GST_PAD_ALWAYS,
+					       raw_caps, NULL);
+  
+  pngenc_src_template = gst_pad_template_new ("src", GST_PAD_SRC,
+					      GST_PAD_ALWAYS,
+					      png_caps, NULL);
+  
+  gst_element_class_add_pad_template (element_class, pngenc_sink_template);
+  gst_element_class_add_pad_template (element_class, pngenc_src_template);
+  gst_element_class_set_details (element_class, &gst_pngenc_details);
+}
+
 static void
 gst_pngenc_class_init (GstPngEncClass *klass)
 {
@@ -104,6 +151,7 @@ static GstPadLinkReturn
 gst_pngenc_sinklink (GstPad *pad, GstCaps *caps)
 {
   GstPngEnc *pngenc;
+  gfloat fps;
 
   pngenc = GST_PNGENC (gst_pad_get_parent (pad));
 
@@ -112,8 +160,15 @@ gst_pngenc_sinklink (GstPad *pad, GstCaps *caps)
 
   gst_caps_get_int (caps, "width", &pngenc->width);
   gst_caps_get_int (caps, "height", &pngenc->height);
+  gst_caps_get_float (caps, "framerate", &fps);
   gst_caps_get_int (caps, "bpp", &pngenc->bpp);
 
+  caps = GST_CAPS_NEW ("png_src",
+		       "video/x-png",
+			 "framerate", GST_PROPS_FLOAT (fps),
+			 "width",     GST_PROPS_INT (pngenc->width),
+			 "height",    GST_PROPS_INT (pngenc->height));
+
   return gst_pad_try_set_caps (pngenc->srcpad, caps);
 }
 
@@ -140,7 +195,7 @@ GstPngEnc *pngenc;
 
   pngenc = (GstPngEnc *) png_get_io_ptr (png_ptr);
 
-  gst_pad_push (pngenc->srcpad, GST_BUFFER (gst_event_new (GST_EVENT_FLUSH)));
+  gst_pad_push (pngenc->srcpad, GST_DATA (gst_event_new (GST_EVENT_FLUSH)));
 }
 
 
@@ -165,8 +220,9 @@ void user_write_data (png_structp png_ptr, png_bytep data, png_uint_32 length)
 }
 
 static void
-gst_pngenc_chain (GstPad *pad, GstBuffer *buf)
+gst_pngenc_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstPngEnc *pngenc;
   gint row_index;
   png_byte *row_pointers[MAX_HEIGHT];
@@ -198,7 +254,7 @@ gst_pngenc_chain (GstPad *pad, GstBuffer *buf)
   /* non-0 return is from a longjmp inside of libpng */
   if (setjmp (pngenc->png_struct_ptr->jmpbuf) != 0)
   {
-    GST_DEBUG (GST_CAT_PLUGIN_INFO, "returning from longjmp");
+    GST_DEBUG ("returning from longjmp");
     png_destroy_write_struct (&pngenc->png_struct_ptr, &pngenc->png_info_ptr);
     return;
   }
@@ -235,10 +291,11 @@ gst_pngenc_chain (GstPad *pad, GstBuffer *buf)
   png_destroy_info_struct (pngenc->png_struct_ptr, &pngenc->png_info_ptr);
   png_destroy_write_struct (&pngenc->png_struct_ptr, (png_infopp) NULL);
 
-  gst_pad_push (pngenc->srcpad, pngenc->buffer_out);
-  /* send NEW MEDIA event, since a frame has been pushed out */
-  event = gst_event_new (GST_EVENT_NEW_MEDIA);
-  gst_pad_send_event (pngenc->srcpad, event);
+  gst_pad_push (pngenc->srcpad, GST_DATA (pngenc->buffer_out));
+
+  /* send EOS event, since a frame has been pushed out */
+  event = gst_event_new (GST_EVENT_EOS);
+  gst_pad_push (pngenc->srcpad, GST_DATA (event));
 
   gst_buffer_unref (buf);
 }
