@@ -18,19 +18,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "gstswfdec.h"
 #include <string.h>
 
 /* elementfactory information */
-static GstElementDetails gst_swfdec_details = {
+static GstElementDetails gst_swfdec_details = GST_ELEMENT_DETAILS (
   "SWF video decoder",
-  "Codec/Video/Decoder",
-  "LGPL",
+  "Codec/Decoder/Video",
   "Uses libswfdec to decode Flash video streams",
-  VERSION,
-  "David Schleef <ds@schleef.org>",
-  "(C) 2002",
-};
+  "David Schleef <ds@schleef.org>"
+);
 
 /* Swfdec signals and args */
 enum {
@@ -40,7 +40,6 @@ enum {
 
 enum {
   ARG_0,
-  ARG_FRAME_RATE,
   /* FILL ME */
 };
 
@@ -50,8 +49,7 @@ GST_PAD_TEMPLATE_FACTORY (video_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "swfdec_videosrc",
-    "video/raw",
-      "format",		GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R','G','B',' ')),
+    "video/x-raw-rgb",
       "width",		GST_PROPS_INT_RANGE (16, 4096),
       "height",		GST_PROPS_INT_RANGE (16, 4096),
       "bpp",		GST_PROPS_INT (24),
@@ -59,7 +57,8 @@ GST_PAD_TEMPLATE_FACTORY (video_template_factory,
       "endianness",	GST_PROPS_INT (G_BIG_ENDIAN),
       "red_mask",	GST_PROPS_INT (0xff0000),
       "green_mask",	GST_PROPS_INT (0x00ff00),
-      "blue_mask",	GST_PROPS_INT (0x0000ff)
+      "blue_mask",	GST_PROPS_INT (0x0000ff),
+      "framerate",	GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT)
   )
 );
 
@@ -69,9 +68,7 @@ GST_PAD_TEMPLATE_FACTORY (audio_template_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "swfdec_audiosrc",
-    "audio/raw",
-      "format",		GST_PROPS_STRING("int"),
-      "law",		GST_PROPS_INT(0),
+    "audio/x-raw-int",
       "endianness",	GST_PROPS_INT(G_BYTE_ORDER),
       "signed",		GST_PROPS_BOOLEAN(TRUE),
       "width",		GST_PROPS_INT(16),
@@ -92,6 +89,7 @@ GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
   )
 );
 
+static void	gst_swfdec_base_init		(gpointer g_class);
 static void	gst_swfdec_class_init		(GstSwfdecClass *klass);
 static void	gst_swfdec_init		(GstSwfdec *swfdec);
 
@@ -128,7 +126,7 @@ gst_swfdec_get_type (void)
   if (!swfdec_type) {
     static const GTypeInfo swfdec_info = {
       sizeof(GstSwfdecClass),      
-      NULL,
+      gst_swfdec_base_init,
       NULL,
       (GClassInitFunc)gst_swfdec_class_init,
       NULL,
@@ -142,6 +140,20 @@ gst_swfdec_get_type (void)
   return swfdec_type;
 }
 
+static void
+gst_swfdec_base_init(gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &gst_swfdec_details);
+
+  gst_element_class_add_pad_template (element_class,
+		  GST_PAD_TEMPLATE_GET (video_template_factory));
+  gst_element_class_add_pad_template (element_class,
+		  GST_PAD_TEMPLATE_GET (audio_template_factory));
+  gst_element_class_add_pad_template (element_class,
+		  GST_PAD_TEMPLATE_GET (sink_template_factory));
+}
 static void
 gst_swfdec_class_init(GstSwfdecClass *klass)
 {
@@ -153,10 +165,6 @@ gst_swfdec_class_init(GstSwfdecClass *klass)
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAME_RATE,
-    g_param_spec_float ("frame_rate","frame_rate","frame_rate",
-                        0.0, 1000.0, 0.0, G_PARAM_READABLE)); 
-  
   gobject_class->set_property 	= gst_swfdec_set_property;
   gobject_class->get_property 	= gst_swfdec_get_property;
   gobject_class->dispose 	= gst_swfdec_dispose;
@@ -171,11 +179,9 @@ static GstCaps *gst_swfdec_videosrc_getcaps(GstPad *pad, GstCaps *caps)
 
   swfdec = GST_SWFDEC (gst_pad_get_parent (pad));
 
-#if GST_VERSION_MINOR > 6
   c = GST_CAPS_NEW (
     "swfdec_videosrc",
-    "video/raw",
-      "format",		GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R','G','B',' ')),
+    "video/x-raw-rgb",
       "width",		GST_PROPS_INT_RANGE (16, 4096),
       "height",		GST_PROPS_INT_RANGE (16, 4096),
       "bpp",		GST_PROPS_INT (24),
@@ -183,47 +189,13 @@ static GstCaps *gst_swfdec_videosrc_getcaps(GstPad *pad, GstCaps *caps)
       "endianness",	GST_PROPS_INT (G_BIG_ENDIAN),
       "red_mask",	GST_PROPS_INT (0xff0000),
       "green_mask",	GST_PROPS_INT (0x00ff00),
-      "blue_mask",	GST_PROPS_INT (0x0000ff)
+      "blue_mask",	GST_PROPS_INT (0x0000ff),
+      "framerate",	GST_PROPS_FLOAT (swfdec->frame_rate)
   );
-#else
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-  c = GST_CAPS_NEW (
-    "swfdec_videosrc",
-    "video/raw",
-      "format",		GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R','G','B',' ')),
-      "width",		GST_PROPS_INT_RANGE (16, 4096),
-      "height",		GST_PROPS_INT_RANGE (16, 4096),
-      "bpp",		GST_PROPS_INT (24),
-      "depth",		GST_PROPS_INT (24),
-      "endianness",	GST_PROPS_INT (G_BIG_ENDIAN),
-      "red_mask",	GST_PROPS_INT (0xff0000),
-      "green_mask",	GST_PROPS_INT (0x00ff00),
-      "blue_mask",	GST_PROPS_INT (0x0000ff)
-  );
-#else
-  c = GST_CAPS_NEW (
-    "swfdec_videosrc",
-    "video/raw",
-      "format",		GST_PROPS_FOURCC (GST_MAKE_FOURCC ('R','G','B',' ')),
-      "width",		GST_PROPS_INT_RANGE (16, 4096),
-      "height",		GST_PROPS_INT_RANGE (16, 4096),
-      "bpp",		GST_PROPS_INT (24),
-      "depth",		GST_PROPS_INT (24),
-      "endianness",	GST_PROPS_INT (G_LITTLE_ENDIAN),
-      "red_mask",	GST_PROPS_INT (0x0000ff),
-      "green_mask",	GST_PROPS_INT (0x00ff00),
-      "blue_mask",	GST_PROPS_INT (0xff0000)
-  );
-#endif
-#endif
 
   if(swfdec->height){
     gst_caps_set(c,"height",GST_PROPS_INT(swfdec->height));
     gst_caps_set(c,"width",GST_PROPS_INT(swfdec->width));
-    
-    /* FIXME: this feels like a hack or bug workaround */
-    GST_PROPS_FLAG_SET(c->properties,GST_PROPS_FIXED);
-    GST_CAPS_FLAG_SET(c,GST_CAPS_FIXED);
   }
 
   return c;
@@ -246,20 +218,20 @@ gst_swfdec_loop(GstElement *element)
 
 	ret = swfdec_decoder_parse(swfdec->state);
 	if(ret==SWF_NEEDBITS){
-		buf = gst_pad_pull(swfdec->sinkpad);
+		buf = GST_BUFFER (gst_pad_pull(swfdec->sinkpad));
 		if(GST_IS_EVENT(buf)){
 			switch (GST_EVENT_TYPE (buf)) {
 			case GST_EVENT_EOS:
-				printf("got eos\n");
+				GST_DEBUG("got eos");
 				break;
 			default:
-				printf("got event\n");
+				GST_DEBUG("got event");
 				break;
 			}
 
 		}else{
 			if(!GST_BUFFER_DATA(buf)){
-				printf("expected non-null buffer\n");
+				GST_DEBUG("expected non-null buffer");
 			}
 			ret = swfdec_decoder_addbits(swfdec->state,
 				GST_BUFFER_DATA(buf), GST_BUFFER_SIZE(buf));
@@ -293,7 +265,7 @@ gst_swfdec_loop(GstElement *element)
 		swfdec->timestamp += swfdec->interval;
 		GST_BUFFER_TIMESTAMP(newbuf) = swfdec->timestamp;
 
-		gst_pad_push(swfdec->videopad, newbuf);
+		gst_pad_push(swfdec->videopad, GST_DATA (newbuf));
 
 		/* audio stuff */
 
@@ -305,15 +277,15 @@ gst_swfdec_loop(GstElement *element)
 			GST_BUFFER_DATA(newbuf) = data;
 			GST_BUFFER_TIMESTAMP(newbuf) = swfdec->timestamp;
 
-			gst_pad_push(swfdec->audiopad, newbuf);
+			gst_pad_push(swfdec->audiopad, GST_DATA (newbuf));
 
 			data = swfdec_decoder_get_sound_chunk(swfdec->state, &len);
 		}
 	}
 
 	if(ret==SWF_EOF){
-		gst_pad_push(swfdec->videopad, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
-		gst_pad_push(swfdec->audiopad, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
+		gst_pad_push(swfdec->videopad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
+		gst_pad_push(swfdec->audiopad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
 	}
 }
 
@@ -351,6 +323,8 @@ gst_swfdec_init (GstSwfdec *swfdec)
   swfdec_decoder_set_colorspace(swfdec->state, SWF_COLORSPACE_RGB888);
 
   GST_FLAG_SET (GST_ELEMENT (swfdec), GST_ELEMENT_EVENT_AWARE);
+
+  swfdec->frame_rate = 0.;
 }
 
 static void
@@ -377,8 +351,6 @@ gst_swfdec_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
         default:
           res = FALSE;
@@ -386,8 +358,6 @@ gst_swfdec_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
         default:
           res = FALSE;
@@ -413,8 +383,6 @@ gst_swfdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
         default:
           res = FALSE;
@@ -422,23 +390,19 @@ gst_swfdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * 6 * (swfdec->width * swfdec->height >> 2) *  
 		  video_rates[swfdec->decoder->frame_rate_code] / GST_SECOND;
 	  break;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * video_rates[swfdec->decoder->frame_rate_code] / GST_SECOND;
 	  break;
         default:
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (video_rates[swfdec->decoder->frame_rate_code] != 0.0) {
 	    *dest_value = src_value * GST_SECOND /
@@ -450,7 +414,7 @@ gst_swfdec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * 6 * (swfdec->width * swfdec->height >> 2);
 	  break;
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value;
 	  break;
         default:
@@ -477,9 +441,6 @@ gst_swfdec_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_TOTAL:
     {
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-          /* fallthrough */
         case GST_FORMAT_TIME:
 	{
 	  int n_frames;
@@ -502,9 +463,6 @@ gst_swfdec_src_query (GstPad *pad, GstQueryType type,
     case GST_QUERY_POSITION:
     {
       switch (*format) {
-        case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-          /* fallthrough */
         default:
           res = FALSE;
           break;
@@ -620,7 +578,7 @@ gst_swfdec_change_state (GstElement *element)
 #if 0
       swfdec->pool = gst_pad_get_bufferpool (swfdec->videopad);
       if (swfdec->pool)
-        GST_INFO (GST_CAT_PLUGIN_INFO, "got pool %p", swfdec->pool);
+        GST_INFO ( "got pool %p", swfdec->pool);
 #endif
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
@@ -675,60 +633,26 @@ gst_swfdec_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
   swfdec = GST_SWFDEC (object);
 
   switch (prop_id) {
-    case ARG_FRAME_RATE:
-      g_value_set_float (value, swfdec->frame_rate);
-      break;
     default:
       break;
   }
 }
 
-static GstCaps *
-swf_type_find(GstBuffer *buf, gpointer private)
-{
-	gchar *data = GST_BUFFER_DATA(buf);
-
-	if((data[0] != 'F' && data[0] != 'C') ||
-	    data[1] != 'W' || data[2] != 'S')return NULL;
-
-	return gst_caps_new("swf_type_find","application/x-shockwave-flash",
-		NULL);
-}
-
-static GstTypeDefinition swftype_definition = 
-	{ "swfdecode/x-shockwave-flash", "application/x-shockwave-flash",
-		".swf .swfl", swf_type_find };
-
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-
-  /* create an elementfactory for the swfdec element */
-  factory = gst_element_factory_new("swfdec",GST_TYPE_SWFDEC,
-                                   &gst_swfdec_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-
-  gst_element_factory_add_pad_template (factory,
-		  GST_PAD_TEMPLATE_GET (video_template_factory));
-  gst_element_factory_add_pad_template (factory,
-		  GST_PAD_TEMPLATE_GET (audio_template_factory));
-  gst_element_factory_add_pad_template (factory,
-		  GST_PAD_TEMPLATE_GET (sink_template_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  type = gst_type_factory_new(&swftype_definition);
-  gst_plugin_add_feature(plugin, GST_PLUGIN_FEATURE(type));
-
-  return TRUE;
+  return gst_element_register (plugin, "swfdec", GST_RANK_PRIMARY, GST_TYPE_SWFDEC);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "swfdec",
-  plugin_init
-};
+  "Uses libswfdec to decode Flash video streams",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
