@@ -488,16 +488,16 @@ static void draw_edges_mmx(uint8_t *buf, int wrap, int width, int height, int w)
     }
 }
 
-#if 0
 #undef HAVE_MMX2
 #define RENAME(a) a ## _MMX
+#define RENAMEl(a) a ## _mmx
 #include "mpegvideo_mmx_template.c"
 
 #define HAVE_MMX2
 #undef RENAME
 #define RENAME(a) a ## _MMX2
+#define RENAMEl(a) a ## _mmx2
 #include "mpegvideo_mmx_template.c"
-#endif
 
 void MPV_common_init_mmx(MpegEncContext *s)
 {
@@ -510,7 +510,6 @@ void MPV_common_init_mmx(MpegEncContext *s)
 
         draw_edges = draw_edges_mmx;
 
-#if 0
         if(dct_algo==FF_DCT_AUTO || dct_algo==FF_DCT_MMX){
             if(mm_flags & MM_MMXEXT){
                 s->dct_quantize= dct_quantize_MMX2;
@@ -518,6 +517,5 @@ void MPV_common_init_mmx(MpegEncContext *s)
                 s->dct_quantize= dct_quantize_MMX;
             }
         }
-#endif
     }
 }
