@@ -17,25 +17,26 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <riff.h>
 
-/*#define debug(format,args...) g_print(format,##args) */
-#define debug(format,args...)
-
 
 gulong gst_riff_fourcc_to_id(gchar *fourcc) {
   g_return_val_if_fail(fourcc != NULL, 0);
 
-  return (fourcc[0] << 0) | (fourcc[1] << 8) |
-         (fourcc[2] << 16) | (fourcc[3] << 24);
+  return GUINT32_FROM_LE((gulong)(fourcc[0] << 0) | (fourcc[1] << 8) |
+         (fourcc[2] << 16) | (fourcc[3] << 24));
 }
 
 gchar *gst_riff_id_to_fourcc(gulong id) {
-  gchar *fourcc = (gchar *)g_malloc(5);
+  static gchar fourcc[5];
 
   g_return_val_if_fail(fourcc != NULL, NULL);
 
+  id = GUINT32_FROM_LE(id);
   fourcc[0] = (id >> 0) & 0xff;
   fourcc[1] = (id >> 8) & 0xff;
   fourcc[2] = (id >> 16) & 0xff;
