@@ -28,51 +28,10 @@
 static GstElementDetails gst_asf_demux_details = {
   "ASF Demuxer",
   "Codec/Demuxer",
-  "LGPL",
   "Demultiplexes ASF Streams",
-  VERSION,
   "Owen Fraser-Green <owen@discobabe.net>",
-  "(C) 2002",
 };
 
-static GstCaps* asf_type_find (GstByteStream *bs, gpointer private);
-
-/* typefactory for 'asf' */
-static GstTypeDefinition asf_type_definition = {
-  "asfdemux_video/asf",
-  "video/x-ms-asf",
-  /* note: asx/wax/wmx are XML files, we don't handle them */
-  ".asf .wma .wmv .wm",
-  asf_type_find,
-};
-
-static GstCaps*
-asf_type_find (GstByteStream *bs, gpointer private)
-{
-  GstCaps *new = NULL;
-  GstBuffer *buf = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 16) == 16) {
-    guint32 uid1 = GUINT32_FROM_LE (((guint32 *) GST_BUFFER_DATA (buf))[0]),
-	    uid2 = GUINT32_FROM_LE (((guint32 *) GST_BUFFER_DATA (buf))[1]),
-	    uid3 = GUINT32_FROM_LE (((guint32 *) GST_BUFFER_DATA (buf))[2]),
-	    uid4 = GUINT32_FROM_LE (((guint32 *) GST_BUFFER_DATA (buf))[3]);
-
-    if (uid1 == 0x75B22630 && uid2 == 0x11CF668E &&
-        uid3 == 0xAA00D9A6 && uid4 == 0x6CCE6200) {
-      new = GST_CAPS_NEW ("asf_type_find",
-			  "video/x-ms-asf",
-			    NULL);
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
-}
-
 GST_PAD_TEMPLATE_FACTORY (sink_factory,
   "sink",
   GST_PAD_SINK,
@@ -82,6 +41,7 @@ GST_PAD_TEMPLATE_FACTORY (sink_factory,
 		  NULL)
 );
 			  
+static void	gst_asf_demux_base_init		(gpointer g_class);
 static void 	gst_asf_demux_class_init	(GstASFDemuxClass *klass);
 static void 	gst_asf_demux_init		(GstASFDemux *asf_demux);
 static gboolean gst_asf_demux_send_event 	(GstElement *element, 
@@ -121,6 +81,10 @@ static gboolean            gst_asf_demux_setup_pad        (GstASFDemux *asf_demu
 							   guint16 id);
 
 static GstElementStateReturn gst_asf_demux_change_state   (GstElement *element);
+static GstCaps * gst_asf_demux_video_caps (guint32 codec_fcc,
+					   asf_stream_video_format *video);
+static GstCaps * gst_asf_demux_audio_caps (guint16 codec_id,
+					   asf_stream_audio *audio, guint8 *extradata);
 
 static GstPadTemplate *videosrctempl, *audiosrctempl;
 static GstElementClass *parent_class = NULL;
@@ -133,7 +97,7 @@ asf_demux_get_type (void)
   if (!asf_demux_type) {
     static const GTypeInfo asf_demux_info = {
       sizeof(GstASFDemuxClass),      
-      NULL,
+      gst_asf_demux_base_init,
       NULL,
       (GClassInitFunc)gst_asf_demux_class_init,
       NULL,
@@ -147,6 +111,63 @@ asf_demux_get_type (void)
   return asf_demux_type;
 }
 
+static void
+gst_asf_demux_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  int i;
+  GstCaps *audcaps = NULL, *vidcaps = NULL, *temp;
+  guint32 vid_list[] = {
+    GST_MAKE_FOURCC('I','4','2','0'),
+    GST_MAKE_FOURCC('Y','U','Y','2'),
+    GST_MAKE_FOURCC('M','J','P','G'),
+    GST_MAKE_FOURCC('D','V','S','D'),
+    GST_MAKE_FOURCC('W','M','V','1'),
+    GST_MAKE_FOURCC('W','M','V','2'),
+    GST_MAKE_FOURCC('M','P','G','4'),
+    GST_MAKE_FOURCC('M','P','4','2'),
+    GST_MAKE_FOURCC('M','P','4','3'),
+    GST_MAKE_FOURCC('D','I','V','3'),
+    GST_MAKE_FOURCC('D','X','5','0'),
+    0 /* end */
+  };
+  gint aud_list[] = {
+    GST_RIFF_WAVE_FORMAT_MPEGL3,
+    GST_RIFF_WAVE_FORMAT_MPEGL12,
+    GST_RIFF_WAVE_FORMAT_PCM,
+    GST_RIFF_WAVE_FORMAT_VORBIS1,
+    GST_RIFF_WAVE_FORMAT_A52,
+    GST_RIFF_WAVE_FORMAT_DIVX_WMAV1,
+    GST_RIFF_WAVE_FORMAT_DIVX_WMAV2,
+    GST_RIFF_WAVE_FORMAT_WMAV9,
+    -1 /* end */
+  };
+
+  for (i = 0; aud_list[i] != -1; i++) {
+    temp = gst_asf_demux_audio_caps (aud_list[i], NULL, NULL);
+    audcaps = gst_caps_append (audcaps, temp);
+  }
+
+  audiosrctempl = gst_pad_template_new ("audio_%02d",
+					GST_PAD_SRC,
+				        GST_PAD_SOMETIMES,
+					audcaps, NULL);
+  for (i = 0; vid_list[i] != 0; i++) {
+    temp = gst_asf_demux_video_caps (vid_list[i], NULL);
+    vidcaps = gst_caps_append (vidcaps, temp);
+  }
+
+  videosrctempl = gst_pad_template_new ("video_%02d",
+					GST_PAD_SRC,
+				      	GST_PAD_SOMETIMES,
+					vidcaps, NULL);
+  gst_element_class_add_pad_template (element_class, audiosrctempl);
+  gst_element_class_add_pad_template (element_class, videosrctempl);
+  gst_element_class_add_pad_template (element_class, 
+		  GST_PAD_TEMPLATE_GET (sink_factory));
+  gst_element_class_set_details (element_class, &gst_asf_demux_details);
+}
+
 static void
 gst_asf_demux_class_init (GstASFDemuxClass *klass) 
 {
@@ -635,12 +656,11 @@ gst_asf_demux_process_stream (GstASFDemux *asf_demux, guint64 *obj_size)
 
   switch (stream_id) {
   case ASF_STREAM_AUDIO:
-
     gst_asf_demux_read_object_header_rest (asf_demux, &ptr, 18);
     audio_object = (asf_stream_audio *)ptr;
     size = GUINT16_FROM_LE (audio_object->size);
 
-    GST_INFO ( "Object is an audio stream with %u bytes of additional data.", size);
+    GST_INFO ("Object is an audio stream with %u bytes of additional data.", size);
 
     if (!gst_asf_demux_add_audio_stream (asf_demux, audio_object, id))
       return FALSE;
@@ -673,6 +693,8 @@ gst_asf_demux_process_stream (GstASFDemux *asf_demux, guint64 *obj_size)
 
       break;
     case ASF_CORRECTION_OFF:
+      GST_INFO ( "Error correction off");
+      gst_bytestream_flush (asf_demux->bs, object->stream_specific_size);
       break;
     default:
       gst_element_error (GST_ELEMENT (asf_demux), "Audio stream using unknown error correction");
@@ -832,6 +854,7 @@ gst_asf_demux_get_stream (GstASFDemux *asf_demux,
 
   /* Base case if we haven't found one at all */
   gst_element_error (GST_ELEMENT (asf_demux), "Segment found for undefined stream: (%d)", id);
+
   return NULL;
 }
 
@@ -972,7 +995,7 @@ gst_asf_demux_handle_sink_event (GstASFDemux *asf_demux,
 				 guint32      remaining)
 {
   GstEventType type;
-  
+
   type = event? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
 
   switch (type) {
@@ -1189,7 +1212,7 @@ gst_asf_demux_identify_guid (GstASFDemux *asf_demux,
 	GST_CAPS_NEW (name,						\
 		      mimetype,						\
 		      "rate",     GST_PROPS_INT (			\
-				    GUINT32_FROM_LE (audio->byte_rate)), \
+				    GUINT32_FROM_LE (audio->sample_rate)), \
 		      "channels", GST_PROPS_INT (			\
 				    GUINT16_FROM_LE (audio->channels)),	\
 		      __VA_ARGS__)					\
@@ -1207,7 +1230,7 @@ gst_asf_demux_identify_guid (GstASFDemux *asf_demux,
 	GST_CAPS_NEW (name,						\
 		      mimetype,						\
 		      "rate",     GST_PROPS_INT (			\
-				    GUINT32_FROM_LE (audio->byte_rate)), \
+				    GUINT32_FROM_LE (audio->sample_rate)), \
 		      "channels", GST_PROPS_INT (			\
 				    GUINT16_FROM_LE (audio->channels)),	\
 		      ##props)						\
@@ -1221,9 +1244,13 @@ gst_asf_demux_identify_guid (GstASFDemux *asf_demux,
 
 static GstCaps *
 gst_asf_demux_audio_caps (guint16 codec_id,
-			  asf_stream_audio *audio)
+			  asf_stream_audio *audio, guint8 *extradata)
 {
   GstCaps *caps = NULL;
+  gint flags1, flags2;
+
+  flags1 = 0;
+  flags2 = 0;
 
   switch (codec_id) {
     case GST_RIFF_WAVE_FORMAT_MPEGL3: /* mp3 */
@@ -1294,15 +1321,54 @@ gst_asf_demux_audio_caps (guint16 codec_id,
       break;
 
     case GST_RIFF_WAVE_FORMAT_DIVX_WMAV1:
-      caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_wmav1",
-				   "audio/x-wma",
-				     "wmaversion", GST_PROPS_INT (1));
+      /* get flags1 and flags2 ripped from ffmpeg (wmadec.c) */
+      if (audio && audio->size >= 4) {
+        flags1 = extradata[0] | (extradata[1] << 8);
+        flags2 = extradata[2] | (extradata[3] << 8);
+      }
+      if (audio != NULL)
+        caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_wmav1",
+                                     "audio/x-wma",
+                                       "wmaversion",  GST_PROPS_INT (1),                        
+                                       "flags1",      GST_PROPS_INT (flags1),
+                                       "flags2",      GST_PROPS_INT (flags2),
+                                       "block_align", GST_PROPS_INT (audio->block_align),
+                                       "bitrate",     GST_PROPS_INT (audio->byte_rate * 8));
+      else
+        caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_wmav1",
+                                     "audio/x-wma",
+	 			       "wmaversion",  GST_PROPS_INT (1),
+                                       "flags1",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                       "flags2",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                       "block_align", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                                       "bitrate",     GST_PROPS_INT_RANGE (0, G_MAXINT)
+                                    );
       break;
 
     case GST_RIFF_WAVE_FORMAT_DIVX_WMAV2:
-      caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_wmav2",
-				   "audio/x-wma",
-				     "wmaversion", GST_PROPS_INT (2));
+      /* get flags1 and flags2 ripped from ffmpeg (wmadec.c) */
+      if (audio && audio->size >= 6) {
+        flags1 = extradata[0] | (extradata[1] << 8) | 
+                (extradata[2] << 16) | (extradata[3] << 24);
+        flags2 = extradata[4] | (extradata[5] << 8);
+      }
+      if (audio != NULL)
+        caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_wmav2",
+				     "audio/x-wma",
+				       "wmaversion",  GST_PROPS_INT (2),
+                                       "flags1",      GST_PROPS_INT (flags1),
+                                       "flags2",      GST_PROPS_INT (flags2),
+                                       "block_align", GST_PROPS_INT (audio->block_align),
+                                       "bitrate",     GST_PROPS_INT (audio->byte_rate * 8));
+     else
+       caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_wmav2",
+	 			    "audio/x-wma",
+				      "wmaversion",  GST_PROPS_INT (2),
+                                      "flags1",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                      "flags2",      GST_PROPS_INT_RANGE (G_MININT, G_MAXINT),
+                                      "block_align", GST_PROPS_INT_RANGE (0, G_MAXINT),
+                                      "bitrate",     GST_PROPS_INT_RANGE (0, G_MAXINT)
+                                   );
       break;
 
     case GST_RIFF_WAVE_FORMAT_WMAV9:
@@ -1329,31 +1395,34 @@ gst_asf_demux_add_audio_stream (GstASFDemux *asf_demux,
   GstCaps        *caps;
   gchar          *name = NULL;
   guint16         size_left = 0;
+  guint8         *extradata=NULL;
 
   size_left = GUINT16_FROM_LE(audio->size);
 
   /* Create the audio pad */
   name = g_strdup_printf ("audio_%02d", asf_demux->num_audio_streams);
+
   src_pad = gst_pad_new_from_template (audiosrctempl, name);
   g_free (name);
 
+  /* Swallow up any left over data */
+  if (size_left) {
+    g_warning ("asfdemux: Audio header contains %d bytes of surplus data", size_left);
+    gst_asf_demux_read_object_header_rest (asf_demux, &extradata, size_left);
+//    gst_bytestream_flush (asf_demux->bs, size_left);
+  }
+
   /* Now set up the standard propertis from the header info */
   caps = gst_asf_demux_audio_caps (GUINT16_FROM_LE(audio->codec_tag),
-				   audio);
+				   audio, extradata);
 
   GST_INFO ("Adding audio stream %u codec %u (0x%x)",
 	    asf_demux->num_video_streams,
 	    GUINT16_FROM_LE(audio->codec_tag),
 	    GUINT16_FROM_LE(audio->codec_tag));
 
-   asf_demux->num_audio_streams++;
+  asf_demux->num_audio_streams++;
 
-   /* Swallow up any left over data */
-   if (size_left) {
-     g_warning ("asfdemux: Audio header contains %d bytes of surplus data", size_left);
-     gst_bytestream_flush (asf_demux->bs, size_left);
-   }
-  
   return gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id);
 }
 
@@ -1508,7 +1577,7 @@ gst_asf_demux_add_video_stream (GstASFDemux *asf_demux,
 	    GUINT32_FROM_LE(video->tag));
   
   asf_demux->num_video_streams++;
-  
+
   return gst_asf_demux_setup_pad (asf_demux, src_pad, caps, id);
 }
 
@@ -1520,7 +1589,7 @@ gst_asf_demux_setup_pad (GstASFDemux *asf_demux,
 			 guint16 id)
 {
   asf_stream_context *stream;
-  
+
   gst_pad_try_set_caps (src_pad, caps_list);
   gst_pad_set_formats_function (src_pad, gst_asf_demux_get_src_formats);
   gst_pad_set_event_mask_function (src_pad, gst_asf_demux_get_src_event_mask);
@@ -1540,6 +1609,7 @@ gst_asf_demux_setup_pad (GstASFDemux *asf_demux,
   GST_INFO ("Adding pad for stream %u",
 	    asf_demux->num_streams);
 
+
   asf_demux->num_streams++;
   
   gst_element_add_pad (GST_ELEMENT (asf_demux), src_pad);
@@ -1547,82 +1617,28 @@ gst_asf_demux_setup_pad (GstASFDemux *asf_demux,
   return TRUE;
 }
 
-
-
-
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-  gint i = 0;
-  GstCaps *audcaps = NULL, *vidcaps = NULL, *temp;
-  guint32 vid_list[] = {
-    GST_MAKE_FOURCC('I','4','2','0'),
-    GST_MAKE_FOURCC('Y','U','Y','2'),
-    GST_MAKE_FOURCC('M','J','P','G'),
-    GST_MAKE_FOURCC('D','V','S','D'),
-    GST_MAKE_FOURCC('W','M','V','1'),
-    GST_MAKE_FOURCC('W','M','V','2'),
-    GST_MAKE_FOURCC('M','P','G','4'),
-    GST_MAKE_FOURCC('M','P','4','2'),
-    GST_MAKE_FOURCC('M','P','4','3'),
-    GST_MAKE_FOURCC('D','I','V','3'),
-    GST_MAKE_FOURCC('D','X','5','0'),
-    0 /* end */
-  };
-  gint aud_list[] = {
-    GST_RIFF_WAVE_FORMAT_MPEGL3,
-    GST_RIFF_WAVE_FORMAT_MPEGL12,
-    GST_RIFF_WAVE_FORMAT_PCM,
-    GST_RIFF_WAVE_FORMAT_VORBIS1,
-    GST_RIFF_WAVE_FORMAT_A52,
-    GST_RIFF_WAVE_FORMAT_DIVX_WMAV1,
-    GST_RIFF_WAVE_FORMAT_DIVX_WMAV2,
-    GST_RIFF_WAVE_FORMAT_WMAV9,
-    -1 /* end */
-  };
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
 
   /* create an elementfactory for the asf_demux element */
-  factory = gst_element_factory_new ("asfdemux",GST_TYPE_ASF_DEMUX,
-                                    &gst_asf_demux_details);
-
-  /* type finding */
-  type = gst_type_factory_new (&asf_type_definition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_NONE);
-
-  for (i = 0; aud_list[i] != -1; i++) {
-    temp = gst_asf_demux_audio_caps (aud_list[i], NULL);
-    audcaps = gst_caps_append (audcaps, temp);
-  }
-  audiosrctempl = gst_pad_template_new ("audio_%02d",
-					GST_PAD_SRC,
-					GST_PAD_SOMETIMES,
-					audcaps, NULL);
-  for (i = 0; vid_list[i] != 0; i++) {
-    temp = gst_asf_demux_video_caps (vid_list[i], NULL);
-    vidcaps = gst_caps_append (vidcaps, temp);
-  }
-  videosrctempl = gst_pad_template_new ("video_%02d",
-					GST_PAD_SRC,
-					GST_PAD_SOMETIMES,
-					vidcaps, NULL);
-  gst_element_factory_add_pad_template (factory, audiosrctempl);
-  gst_element_factory_add_pad_template (factory, videosrctempl);
-  gst_element_factory_add_pad_template (factory,
-	GST_PAD_TEMPLATE_GET (sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "asfdemux", GST_RANK_NONE, GST_TYPE_ASF_DEMUX))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "asfdemux",
-  plugin_init
-};
+  "Demuxes ASF streams",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
