@@ -18,6 +18,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <math.h>
 
@@ -27,15 +30,12 @@
 #include <gst/resample/resample.h>
 
 /* elementfactory information */
-static GstElementDetails audioscale_details = {
+static GstElementDetails gst_audioscale_details = GST_ELEMENT_DETAILS (
   "Audio scaler",
-  "Filter/Audio",
-  "LGPL",
-  "Audio resampler",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2000",
-};
+  "Filter/Converter/Audio",
+  "Resample audio",
+  "David Schleef <ds@schleef.org>"
+);
 
 /* Audioscale signals and args */
 enum {
@@ -45,43 +45,28 @@ enum {
 
 enum {
   ARG_0,
-  ARG_FREQUENCY,
   ARG_FILTERLEN,
   ARG_METHOD,
   /* FILL ME */
 };
 
-static GstPadTemplate *
-sink_template (void)
-{
-  static GstPadTemplate *template = NULL;
+GST_PAD_TEMPLATE_FACTORY (sink_factory,
+  "sink",
+  GST_PAD_SINK,
+  GST_PAD_ALWAYS,
+  gst_caps_new ("audioscale_sink",
+		"audio/x-raw-int",
+		  GST_AUDIO_INT_PAD_TEMPLATE_PROPS)
+);
 
-  if (!template) {
-    template = gst_pad_template_new ("sink",
-				    GST_PAD_SINK,
-				    GST_PAD_ALWAYS,
-				    gst_caps_new
-				    ("audioscale_sink",
-				     "audio/raw", GST_AUDIO_INT_PAD_TEMPLATE_PROPS), NULL);
-  }
-  return template;
-}
-
-static GstPadTemplate *
-src_template (void)
-{
-  static GstPadTemplate *template = NULL;
-
-  if (!template) {
-    template = gst_pad_template_new ("src",
-				    GST_PAD_SRC,
-				    GST_PAD_ALWAYS,
-				    gst_caps_new
-				    ("audioscale_src",
-				     "audio/raw", GST_AUDIO_INT_PAD_TEMPLATE_PROPS), NULL);
-  }
-  return template;
-}
+GST_PAD_TEMPLATE_FACTORY (src_factory,
+  "src",
+  GST_PAD_SRC,
+  GST_PAD_ALWAYS,
+  gst_caps_new ("audioscale_src",
+		"audio/x-raw-int",
+		  GST_AUDIO_INT_PAD_TEMPLATE_PROPS)
+);
 
 #define GST_TYPE_AUDIOSCALE_METHOD (gst_audioscale_method_get_type())
 static GType
@@ -101,10 +86,11 @@ gst_audioscale_method_get_type (void)
   return audioscale_method_type;
 }
 
+static void     gst_audioscale_base_init        (gpointer g_class);
 static void	gst_audioscale_class_init	(AudioscaleClass *klass);
 static void	gst_audioscale_init		(Audioscale *audioscale);
 
-static void	gst_audioscale_chain		(GstPad *pad, GstBuffer *buf);
+static void	gst_audioscale_chain		(GstPad *pad, GstData *_data);
 
 static void gst_audioscale_set_property (GObject * object, guint prop_id,
 					 const GValue * value, GParamSpec * pspec);
@@ -122,7 +108,8 @@ audioscale_get_type (void)
 
   if (!audioscale_type) {
     static const GTypeInfo audioscale_info = {
-      sizeof(AudioscaleClass),      NULL,
+      sizeof(AudioscaleClass),
+      gst_audioscale_base_init,
       NULL,
       (GClassInitFunc)gst_audioscale_class_init,
       NULL,
@@ -136,6 +123,19 @@ audioscale_get_type (void)
   return audioscale_type;
 }
 
+static void
+gst_audioscale_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (gstelement_class,
+		  GST_PAD_TEMPLATE_GET (src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+		  GST_PAD_TEMPLATE_GET (sink_factory));
+
+  gst_element_class_set_details (gstelement_class, &gst_audioscale_details);
+}
+
 static void
 gst_audioscale_class_init (AudioscaleClass *klass)
 {
@@ -145,48 +145,133 @@ gst_audioscale_class_init (AudioscaleClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_FREQUENCY,
-        g_param_spec_int ("frequency","frequency","frequency",
-                          0,G_MAXINT,44100,G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
+  gobject_class->set_property = gst_audioscale_set_property;
+  gobject_class->get_property = gst_audioscale_get_property;
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FILTERLEN,
 	g_param_spec_int ("filter_length", "filter_length", "filter_length",
                           0, G_MAXINT, 16, G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METHOD,
 	g_param_spec_enum ("method", "method", "method", GST_TYPE_AUDIOSCALE_METHOD,
                            RESAMPLE_SINC, G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
+}
 
-  gobject_class->set_property = gst_audioscale_set_property;
-  gobject_class->get_property = gst_audioscale_get_property;
+static GstCaps *
+gst_audioscale_getcaps (GstPad *pad, GstCaps *caps)
+{
+  Audioscale *audioscale;
+  GstCaps *peercaps;
 
+  audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+
+  if (pad == audioscale->srcpad){
+    peercaps = gst_pad_get_allowed_caps (audioscale->sinkpad);
+  }else{
+    peercaps = gst_pad_get_allowed_caps (audioscale->srcpad);
+  }
+
+  if(peercaps == GST_CAPS_NONE){
+    return GST_CAPS_NONE;
+  }
+
+  caps = gst_caps_copy (peercaps);
+#if 1
+  /* we do this hack, because the audioscale lib doesn't handle
+   * rate conversions larger than a factor of 2 */
+  if(gst_caps_has_property_typed(caps, "rate", GST_PROPS_INT_RANGE_TYPE)){
+    int rate_min, rate_max;
+
+    gst_props_entry_get_int_range (gst_props_get_entry(caps->properties, "rate"),
+	&rate_min, &rate_max);
+    gst_caps_set (caps, "rate", GST_PROPS_INT_RANGE((rate_min+1)/2,
+	  rate_max*2));
+  }else{
+    int rate;
+
+    gst_caps_get_int (caps, "rate", &rate);
+    gst_caps_set (caps, "rate", GST_PROPS_INT_RANGE((rate+1)/2,rate*2));
+  }
+#else
+  gst_caps_set (caps, "rate", GST_PROPS_INT_RANGE(4000,96000));
+#endif
+
+  return caps;
 }
 
 static GstPadLinkReturn
-gst_audioscale_sinkconnect (GstPad * pad, GstCaps * caps)
+gst_audioscale_sink_link (GstPad * pad, GstCaps * caps)
 {
   Audioscale *audioscale;
   resample_t *r;
-  GstCaps *newcaps;
+  GstCaps *caps1;
+  GstCaps *caps2;
+  GstCaps *peercaps;
   gint rate;
+  int ret;
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
   r = audioscale->resample;
 
+  if (!GST_CAPS_IS_FIXED (caps)){
+    return GST_PAD_LINK_DELAYED;
+  }
+
+  ret = gst_pad_try_set_caps (audioscale->srcpad, caps);
+
+  if(ret == GST_PAD_LINK_OK || ret == GST_PAD_LINK_DONE){
+    audioscale->passthru = TRUE;
+    return ret;
+  }
+
+  audioscale->passthru = FALSE;
+
   gst_caps_get_int (caps, "rate",     &rate);
   gst_caps_get_int (caps, "channels", &r->channels);
 
   r->i_rate = rate;
-  
   resample_reinit(r);
-  
-  newcaps = gst_caps_copy (caps);
-  gst_caps_set (newcaps, "rate", GST_PROPS_INT_TYPE, audioscale->targetfrequency, NULL);
 
-  if (GST_CAPS_IS_FIXED (caps))
-    return gst_pad_try_set_caps (audioscale->srcpad, newcaps);
-  else
-    return GST_PAD_LINK_DELAYED;
+  peercaps = gst_pad_get_allowed_caps (audioscale->srcpad);
+
+  caps1 = gst_caps_copy (caps);
+#if 1
+  /* we do this hack, because the audioscale lib doesn't handle
+   * rate conversions larger than a factor of 2 */
+  if(gst_caps_has_property_typed(caps1, "rate", GST_PROPS_INT_RANGE_TYPE)){
+    int rate_min, rate_max;
+
+    gst_props_entry_get_int_range (gst_props_get_entry(caps1->properties, "rate"),
+	&rate_min, &rate_max);
+    gst_caps_set (caps1, "rate", GST_PROPS_INT_RANGE((rate_min+1)/2,
+	  rate_max*2));
+  }else{
+    gst_caps_get_int (caps1, "rate", &rate);
+    gst_caps_set (caps1, "rate", GST_PROPS_INT_RANGE((rate+1)/2,rate*2));
+  }
+#else
+  gst_caps_set (caps1, "rate", GST_PROPS_INT_RANGE(4000,96000));
+#endif
+  caps2 = gst_caps_intersect(caps1, peercaps);
+  gst_caps_unref(caps1);
+
+  if(caps2 == GST_CAPS_NONE){
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  if (GST_CAPS_IS_FIXED (caps2)) {
+    ret = gst_pad_try_set_caps (audioscale->srcpad, caps2);
+    gst_caps_get_int (caps, "rate",     &rate);
+    r->o_rate = rate;
+    audioscale->targetfrequency = rate;
+    resample_reinit(r);
+    return ret;
+  }
+  
+  gst_caps_unref (caps2);
+  return GST_PAD_LINK_DELAYED;
 }
 
 static void *
@@ -208,14 +293,18 @@ gst_audioscale_init (Audioscale *audioscale)
 {
   resample_t *r;
 
-  audioscale->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_template), "sink");
+  audioscale->sinkpad = gst_pad_new_from_template (
+	GST_PAD_TEMPLATE_GET (sink_factory), "sink");
   gst_element_add_pad(GST_ELEMENT(audioscale),audioscale->sinkpad);
   gst_pad_set_chain_function(audioscale->sinkpad,gst_audioscale_chain);
-  gst_pad_set_link_function (audioscale->sinkpad, gst_audioscale_sinkconnect);
+  gst_pad_set_link_function (audioscale->sinkpad, gst_audioscale_sink_link);
+  gst_pad_set_getcaps_function (audioscale->sinkpad, gst_audioscale_getcaps);
 
-  audioscale->srcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (src_template), "src");
+  audioscale->srcpad = gst_pad_new_from_template (
+	GST_PAD_TEMPLATE_GET (src_factory), "src");
 
   gst_element_add_pad(GST_ELEMENT(audioscale),audioscale->srcpad);
+  gst_pad_set_getcaps_function (audioscale->srcpad, gst_audioscale_getcaps);
 
   r = g_new0(resample_t,1);
   audioscale->resample = r;
@@ -236,8 +325,9 @@ gst_audioscale_init (Audioscale *audioscale)
 }
 
 static void
-gst_audioscale_chain (GstPad *pad, GstBuffer *buf)
+gst_audioscale_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   Audioscale *audioscale;
   guchar *data;
   gulong size;
@@ -247,17 +337,22 @@ gst_audioscale_chain (GstPad *pad, GstBuffer *buf)
   g_return_if_fail(buf != NULL);
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+  if (audioscale->passthru){
+    gst_pad_push (audioscale->srcpad, GST_DATA (buf));
+    return;
+  }
+
   data = GST_BUFFER_DATA(buf);
   size = GST_BUFFER_SIZE(buf);
 
-  GST_DEBUG (0,
+  GST_DEBUG (
 	     "gst_audioscale_chain: got buffer of %ld bytes in '%s'\n",
 	     size, gst_element_get_name (GST_ELEMENT (audioscale)));
 
 
   resample_scale (audioscale->resample, data, size);
 
-  gst_pad_push (audioscale->srcpad, audioscale->outbuf);
+  gst_pad_push (audioscale->srcpad, GST_DATA (audioscale->outbuf));
 
   gst_buffer_unref (buf);
 }
@@ -275,13 +370,9 @@ gst_audioscale_set_property (GObject * object, guint prop_id,
   r = src->resample;
 
   switch (prop_id) {
-    case ARG_FREQUENCY:
-      src->targetfrequency = g_value_get_int (value);
-      r->o_rate = src->targetfrequency;
-      break;
     case ARG_FILTERLEN:
       r->filter_length = g_value_get_int (value);
-      GST_DEBUG_ELEMENT (0, GST_ELEMENT(src), "new filter length %d\n", r->filter_length);
+      GST_DEBUG_OBJECT (GST_ELEMENT(src), "new filter length %d\n", r->filter_length);
       break;
     case ARG_METHOD:
       r->method = g_value_get_enum (value);
@@ -304,9 +395,6 @@ gst_audioscale_get_property (GObject *object, guint prop_id, GValue *value, GPar
   r = src->resample;
 
   switch (prop_id) {
-    case ARG_FREQUENCY:
-      g_value_set_int (value, src->targetfrequency);
-      break;
     case ARG_FILTERLEN:
       g_value_set_int (value, r->filter_length);
       break;
@@ -321,25 +409,30 @@ gst_audioscale_get_property (GObject *object, guint prop_id, GValue *value, GPar
 
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the audioscale element */
-  factory = gst_element_factory_new ("audioscale", GST_TYPE_AUDIOSCALE, &audioscale_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
   /* load support library */
   if (!gst_library_load ("gstresample"))
     return FALSE;
 
+  if (!gst_element_register (plugin, "audioscale", GST_RANK_NONE,
+        GST_TYPE_AUDIOSCALE)) {
+    return FALSE;
+  }
+
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "audioscale",
-  plugin_init
-};
+  "Resamples audio",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
+
