@@ -25,13 +25,9 @@
 #define __GST_INT2FLOAT_H__
 
 
-#include <config.h>
 #include <gst/gst.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GST_TYPE_INT2FLOAT \
   (gst_int2float_get_type())
@@ -54,14 +50,14 @@ struct _GstInt2Float {
 
   GstPad *sinkpad;
   GSList *srcpads;
-  
+
   gint numsrcpads;
   GstBuffer **buffers;
   gfloat **data_out;
   GstBufferPool *buffer_pool;
 
   gint64 offset;
-  
+
   GstCaps *intcaps;
   GstCaps *floatcaps;
   gint channels, rate;
@@ -79,9 +75,7 @@ struct _GstInt2FloatClass {
 
 GType gst_int2float_get_type(void);
 gboolean gst_int2float_factory_init (GstPlugin *plugin);
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
 
+G_END_DECLS
 
 #endif /* __GST_INT2FLOAT_H__ */
