@@ -19,11 +19,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
 #include "alaw-decode.h"
 
 extern GstPadTemplate *alawdec_src_template, *alawdec_sink_template;
 
+/* elementfactory information */
+static GstElementDetails alawdec_details = {
+  "A Law to PCM conversion",
+  "Codec/Decoder/Audio",
+  "Convert 8bit A law to 16bit PCM",
+  "Zaheer Merali <zaheer@bellworldwide.net>"
+};
 
 /* Stereo signals and args */
 enum {
@@ -36,12 +46,13 @@ enum {
 };
 
 static void		gst_alawdec_class_init		(GstALawDecClass *klass);
+static void		gst_alawdec_base_init		(GstALawDecClass *klass);
 static void		gst_alawdec_init			(GstALawDec *alawdec);
 
 static void		gst_alawdec_set_property			(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void		gst_alawdec_get_property			(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void		gst_alawdec_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_alawdec_chain			(GstPad *pad, GstData *_data);
 
 
 static GstElementClass *parent_class = NULL;
@@ -86,9 +97,7 @@ alawdec_link (GstPad *pad, GstCaps *caps)
   
   tempcaps = GST_CAPS_NEW (
 	      "alawdec_src_caps",
-	      "audio/raw",
-          "format",   GST_PROPS_STRING ("int"),
-          "law",      GST_PROPS_INT (0),
+	      "audio/x-raw-int",
           "depth",    GST_PROPS_INT (16),
           "width",    GST_PROPS_INT (16),
           "signed",   GST_PROPS_BOOLEAN (TRUE),
@@ -106,7 +115,8 @@ gst_alawdec_get_type(void) {
 
   if (!alawdec_type) {
     static const GTypeInfo alawdec_info = {
-      sizeof(GstALawDecClass),      NULL,
+      sizeof(GstALawDecClass),
+      (GBaseInitFunc)gst_alawdec_base_init,
       NULL,
       (GClassInitFunc)gst_alawdec_class_init,
       NULL,
@@ -120,6 +130,16 @@ gst_alawdec_get_type(void) {
   return alawdec_type;
 }
 
+static void
+gst_alawdec_base_init (GstALawDecClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class, alawdec_src_template);
+  gst_element_class_add_pad_template (element_class, alawdec_sink_template);
+  gst_element_class_set_details (element_class, &alawdec_details);
+}
+
 static void
 gst_alawdec_class_init (GstALawDecClass *klass)
 {
@@ -148,8 +168,9 @@ gst_alawdec_init (GstALawDec *alawdec)
 }
 
 static void
-gst_alawdec_chain (GstPad *pad,GstBuffer *buf)
+gst_alawdec_chain (GstPad *pad,GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstALawDec *alawdec;
   gint16 *linear_data;
   guint8 *alaw_data;
@@ -177,7 +198,7 @@ gst_alawdec_chain (GstPad *pad,GstBuffer *buf)
   }
   
   gst_buffer_unref(buf);
-  gst_pad_push(alawdec->srcpad,outbuf);
+  gst_pad_push(alawdec->srcpad,GST_DATA (outbuf));
 }
 
 static void
