@@ -25,6 +25,10 @@
      Olivier Lapicque <olivierl@jps.net>  
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "libmodplug/stdafx.h"
 #include "libmodplug/sndfile.h"
 
@@ -37,14 +41,9 @@
 /* elementfactory information */
 GstElementDetails modplug_details = {
   "ModPlug",
-  "Codec/Audio/Decoder",
-  "LGPL",
+  "Codec/Decoder/Audio",
   "Module decoder based on modplug engine",
-  VERSION,
-  "Jeremy SIMON <jsimon13@yahoo.fr>\n"
-  "Kenton Varda <temporal@gauge3d.org>\n"
-  "Olivier Lapicque <olivierl@jps.net>",
-  "(C) 2001"
+  "Jeremy SIMON <jsimon13@yahoo.fr>"
 };
 
 
@@ -100,7 +99,7 @@ enum {
   MODPLUG_STATE_PLAY_TUNE = 3,
 };
 
-
+static void	gst_modplug_base_init		(GstModPlugClass *klass);
 static void	gst_modplug_class_init		(GstModPlugClass *klass);
 static void	gst_modplug_init		(GstModPlug *filter);
 static void	gst_modplug_set_property 	(GObject *object,
@@ -129,50 +128,6 @@ static GstElementStateReturn
 
 static GstElementClass *parent_class = NULL;
 
-static GstCaps* 
-modplug_type_find (GstByteStream *bs, gpointer priv) 
-{
-  GstBuffer *buf = NULL;
-  GstCaps *newc = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 75) == 75) {
-    if (MOD_CheckType (buf)     ||
-        Mod_669_CheckType (buf) ||
-        Amf_CheckType (buf)     ||
-        Dsm_CheckType (buf)     ||
-        Fam_CheckType (buf)     ||
-        Gdm_CheckType (buf)     ||
-        Imf_CheckType (buf)     ||
-        It_CheckType (buf)      ||
-        M15_CheckType (buf)     ||
-#if 0
-        Med_CheckType (buf)     || /* FIXME */
-#endif
-        Mtm_CheckType (buf)     ||
-        Okt_CheckType (buf)     ||
-        S3m_CheckType (buf)     ||
-        Xm_CheckType (buf)) {
-      newc = GST_CAPS_NEW ("modplug_type_find",
-			   "audio/x-mod",
-			     NULL);
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return newc;
-}
-
-static GstTypeDefinition modplug_definitions[] = {
-  { "modplug_audio/mod", "audio/x-mod",
-    ".mod .sam .med .stm .mtm .669 .ult .far .amf "
-    ".dsm .imf .gdm .stx .okt .xm .it .s3m",
-    modplug_type_find },
-  { NULL, NULL, NULL, NULL }
-};
-
 GType
 gst_modplug_get_type(void) {
   static GType modplug_type = 0;
@@ -180,7 +135,7 @@ gst_modplug_get_type(void) {
   if (!modplug_type) {
     static const GTypeInfo modplug_info = {
       sizeof(GstModPlugClass),
-      NULL,
+      (GBaseInitFunc)gst_modplug_base_init,
       NULL,
       (GClassInitFunc)gst_modplug_class_init,
       NULL,
@@ -195,6 +150,17 @@ gst_modplug_get_type(void) {
   return modplug_type;
 }
 
+static void
+gst_modplug_base_init (GstModPlugClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+	GST_PAD_TEMPLATE_GET (modplug_sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+	GST_PAD_TEMPLATE_GET (modplug_src_template_factory));
+  gst_element_class_set_details (element_class, &modplug_details);
+}
 
 static void
 gst_modplug_class_init (GstModPlugClass *klass)
@@ -856,44 +822,26 @@ gst_modplug_get_property (GObject *object, guint id, GValue *value, GParamSpec *
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;  
-	guint i;
-  
   /* this filter needs the bytestream package */
-/*  if (!gst_library_load ("gstbytestream"))
-    return FALSE;*/
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
   
-  factory = gst_element_factory_new ("modplug", GST_TYPE_MODPLUG, &modplug_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
- 
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (modplug_sink_template_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (modplug_src_template_factory));
-
-  i = 0;
-  while (modplug_definitions[i].name) {
-    GstTypeFactory *type;
-
-    type = gst_type_factory_new (&modplug_definitions[i]);
-    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-    i++;
-  }
-  
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));	
-
-  return TRUE;
+  return gst_element_register (plugin, "modplug",
+			       GST_RANK_PRIMARY, GST_TYPE_MODPLUG);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
-  "modplug",
-  plugin_init
-};
-
- 
-  
-
+  "monoscope",
+  "Monoscope visualization",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  "(c) 2001 Kenton Varda <temporal@gauge3d.org>\n"
+  "(c) 2001 Olivier Lapicque <olivierl@jps.net>",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
