@@ -23,6 +23,10 @@ psy.c
  * 8/07/92  Mike Coleman        Bug fix, read_absthr()                *
  **********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "common.h"
 #include "encoder.h"
 
@@ -384,18 +388,18 @@ temp2=r[chn][new][j] * sin((double) phi[j]) - r_prime * sin((double) phi_prime);
 /* used at the beginning of this function to simulate "automatic"        */
 /* variables placed on the stack.                                        */
 
- mpegaudio_mem_free((void **) &grouped_c);
- mpegaudio_mem_free((void **) &grouped_e);
- mpegaudio_mem_free((void **) &nb);
- mpegaudio_mem_free((void **) &cb);
- mpegaudio_mem_free((void **) &ecb);
- mpegaudio_mem_free((void **) &bc);
- mpegaudio_mem_free((void **) &wsamp_r);
- mpegaudio_mem_free((void **) &wsamp_i);
- mpegaudio_mem_free((void **) &phi);
- mpegaudio_mem_free((void **) &energy);
- mpegaudio_mem_free((void **) &c);
- mpegaudio_mem_free((void **) &fthr);
- mpegaudio_mem_free((void **) &snrtmp);
+ mpegaudio_mem_free(grouped_c);
+ mpegaudio_mem_free(grouped_e);
+ mpegaudio_mem_free(nb);
+ mpegaudio_mem_free(cb);
+ mpegaudio_mem_free(ecb);
+ mpegaudio_mem_free(bc);
+ mpegaudio_mem_free(wsamp_r);
+ mpegaudio_mem_free(wsamp_i);
+ mpegaudio_mem_free(phi);
+ mpegaudio_mem_free(energy);
+ mpegaudio_mem_free(c);
+ mpegaudio_mem_free(fthr);
+ mpegaudio_mem_free(snrtmp);
 }
 
