@@ -97,20 +97,8 @@ gst_qtdemux_details =
 {
   "QuickTime Demuxer",
   "Codec/Demuxer",
-  "LGPL",
   "Demultiplex a QuickTime file into audio and video streams",
-  VERSION,
-  "David Schleef <ds@schleef.org>",
-  "(C) 2003",
-};
-
-static GstCaps* quicktime_type_find (GstByteStream *bs, gpointer private);
-
-static GstTypeDefinition quicktimedefinition = {
-  "qtdemux_video/quicktime",
-  "video/quicktime",
-  ".mov",
-  quicktime_type_find,
+  "David Schleef <ds@schleef.org>"
 };
 
 enum {
@@ -136,6 +124,7 @@ static GstPadTemplate *videosrctempl, *audiosrctempl;
 static GstElementClass *parent_class = NULL;
 
 static void gst_qtdemux_class_init (GstQTDemuxClass *klass);
+static void gst_qtdemux_base_init (GstQTDemuxClass *klass);
 static void gst_qtdemux_init (GstQTDemux *quicktime_demux);
 static GstElementStateReturn gst_qtdemux_change_state(GstElement *element);
 static void gst_qtdemux_loop_header (GstElement *element);
@@ -155,7 +144,8 @@ static GType gst_qtdemux_get_type (void)
 
   if (!qtdemux_type) {
     static const GTypeInfo qtdemux_info = {
-      sizeof(GstQTDemuxClass), NULL, NULL,
+      sizeof(GstQTDemuxClass),
+      (GBaseInitFunc)gst_qtdemux_base_init, NULL,
       (GClassInitFunc)gst_qtdemux_class_init,
       NULL, NULL, sizeof(GstQTDemux), 0,
       (GInstanceInitFunc)gst_qtdemux_init,
@@ -165,6 +155,17 @@ static GType gst_qtdemux_get_type (void)
   return qtdemux_type;
 }
 
+static void gst_qtdemux_base_init (GstQTDemuxClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (sink_templ));
+  gst_element_class_add_pad_template (element_class, videosrctempl);
+  gst_element_class_add_pad_template (element_class, audiosrctempl);
+  gst_element_class_set_details (element_class, &gst_qtdemux_details);
+}
+
 static void gst_qtdemux_class_init (GstQTDemuxClass *klass) 
 {
   GObjectClass *gobject_class;
@@ -186,37 +187,9 @@ gst_qtdemux_init (GstQTDemux *qtdemux)
   gst_element_add_pad (GST_ELEMENT (qtdemux), qtdemux->sinkpad);
 }
 
-static GstCaps*
-quicktime_type_find (GstByteStream *bs, gpointer private)
-{
-  GstBuffer *buf = NULL;
-  GstCaps *new = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 8) == 8) {
-    gchar *data = GST_BUFFER_DATA (buf);
-
-    if (!strncmp (&data[4], "wide", 4) ||
-        !strncmp (&data[4], "moov", 4) ||
-        !strncmp (&data[4], "mdat", 4) ||
-        !strncmp (&data[4], "free", 4)) {
-      new = GST_CAPS_NEW ("quicktime_type_find",
-			  "video/quicktime",
-			    NULL);
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
-}
-
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
   GstCaps *audiocaps = NULL, *videocaps = NULL, *temp;
   const guint32 audio_fcc[] = {
     /* FILLME */
@@ -227,10 +200,8 @@ plugin_init (GModule *module, GstPlugin *plugin)
   };
   gint i;
 
-  factory = gst_element_factory_new ("qtdemux", GST_TYPE_QTDEMUX,
-                                     &gst_qtdemux_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
 
   for (i = 0; audio_fcc[i] != 0; i++) {
     temp = qtdemux_audio_caps (NULL, audio_fcc[i]);
@@ -250,24 +221,22 @@ plugin_init (GModule *module, GstPlugin *plugin)
 					GST_PAD_SOMETIMES,
 					videocaps, NULL);
 
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_templ));
-  gst_element_factory_add_pad_template (factory, videosrctempl);
-  gst_element_factory_add_pad_template (factory, audiosrctempl);
-
-  type = gst_type_factory_new (&quicktimedefinition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "qtdemux",
+			       GST_RANK_NONE, GST_TYPE_QTDEMUX);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "qtdemux",
-  plugin_init
-};
+  "Quicktime stream demuxer",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
 
 static gboolean gst_qtdemux_handle_sink_event (GstQTDemux *qtdemux)
 {
@@ -365,6 +334,7 @@ static void gst_qtdemux_loop_header (GstElement *element)
     GST_DEBUG ("length %08x",length);
     fourcc = GUINT32_FROM_LE(*(guint32 *)(data+4));
     GST_DEBUG ("fourcc " GST_FOURCC_FORMAT, GST_FOURCC_ARGS(fourcc));
+
     if(length==0){
       length = gst_bytestream_length(qtdemux->bs) - cur_offset;
     }
@@ -383,6 +353,8 @@ static void gst_qtdemux_loop_header (GstElement *element)
       case GST_MAKE_FOURCC('m','d','a','t'):
       case GST_MAKE_FOURCC('f','r','e','e'):
       case GST_MAKE_FOURCC('w','i','d','e'):
+      case GST_MAKE_FOURCC('P','I','C','T'):
+      case GST_MAKE_FOURCC('p','n','o','t'):
         break;
       case GST_MAKE_FOURCC('m','o','o','v'):
       {
@@ -851,8 +823,8 @@ static void qtdemux_parse(GstQTDemux *qtdemux, GNode *node, void *buffer, int le
 
   type = qtdemux_type_get(fourcc);
   
-  //g_print("parsing '" GST_FOURCC_FORMAT "', length=%d\n",
-      //GST_FOURCC_ARGS(fourcc), node_length);
+  /*g_print("parsing '" GST_FOURCC_FORMAT "', length=%d\n",
+      GST_FOURCC_ARGS(fourcc), node_length);*/
 
   if(type->flags & QT_CONTAINER){
     void *buf;
@@ -1266,8 +1238,11 @@ static void qtdemux_parse_tree(GstQTDemux *qtdemux)
   trak = qtdemux_tree_get_child_by_type(qtdemux->moov_node, FOURCC_trak);
   qtdemux_parse_trak(qtdemux, trak);
 
-  trak = qtdemux_tree_get_sibling_by_type(trak, FOURCC_trak);
-  if(trak)qtdemux_parse_trak(qtdemux, trak);
+/*  trak = qtdemux_tree_get_sibling_by_type(trak, FOURCC_trak);
+  if(trak)qtdemux_parse_trak(qtdemux, trak);*/
+
+  while ((trak = qtdemux_tree_get_sibling_by_type(trak, FOURCC_trak)) != NULL)
+    qtdemux_parse_trak(qtdemux, trak);
 }
 
 static void qtdemux_parse_trak(GstQTDemux *qtdemux, GNode *trak)
@@ -1547,19 +1522,19 @@ static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc)
   switch(fourcc){
     case GST_MAKE_FOURCC('j','p','e','g'):
       /* JPEG */
-      return GST_CAPS_NEW("jpeg_caps","video/x-jpeg",
+      return GST_CAPS_NEW("jpeg_caps","image/jpeg",
           "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
           "width", GST_PROPS_INT_RANGE (16, 4096),
           "height", GST_PROPS_INT_RANGE (16, 4096));
     case GST_MAKE_FOURCC('m','j','p','a'):
       /* Motion-JPEG (format A) */
-      return GST_CAPS_NEW("mjpa_caps","video/x-jpeg",
+      return GST_CAPS_NEW("mjpa_caps","image/jpeg",
           "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
           "width", GST_PROPS_INT_RANGE (16, 4096),
           "height", GST_PROPS_INT_RANGE (16, 4096));
     case GST_MAKE_FOURCC('m','j','p','b'):
       /* Motion-JPEG (format B) */
-      return GST_CAPS_NEW("mjpb_caps","video/x-jpeg",
+      return GST_CAPS_NEW("mjpb_caps","image/jpeg",
           "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT),
           "width", GST_PROPS_INT_RANGE (16, 4096),
           "height", GST_PROPS_INT_RANGE (16, 4096));
@@ -1728,6 +1703,7 @@ static GstCaps *qtdemux_audio_caps(GstQTDemux *qtdemux, guint32 fourcc)
 	  NULL);
     case 0x6d730055:
       /* MPEG layer 3, CBR only (pre QT4.1) */
+    case 0x5500736d:
     case GST_MAKE_FOURCC('.','m','p','3'):
       /* MPEG layer 3, CBR & VBR (QT4.1 and later) */
       return GST_CAPS_NEW("_mp3_caps","audio/mpeg",
