@@ -25,7 +25,6 @@
 #define __GST_SINESRC_H__
 
 
-#include <config.h>
 #include <gst/gst.h>
 
 #ifdef __cplusplus
@@ -62,7 +61,6 @@ struct _GstSineSrc {
   gfloat freq;
   
   /* lookup table data */
-  gdouble accumulator;
   gfloat *table_data;
   gdouble table_pos;
   gdouble table_inc;
@@ -83,6 +81,7 @@ struct _GstSineSrc {
 
   gboolean newcaps;
 
+  double accumulator;
 };
 
 struct _GstSineSrcClass {
