@@ -18,25 +18,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 /*#define DEBUG_ENABLED */
 #include <gstvideofilter.h>
 
 
 
-#if 0
-/* elementfactory information */
-static GstElementDetails videofilter_details = {
-  "Video scaler",
-  "Filter/Video",
-  "LGPL",
-  "Resizes video",
-  VERSION,
-  "David Schleef <ds@schleef.org>",
-  "(C) 2003",
-};
-#endif
-
 /* GstVideofilter signals and args */
 enum {
   /* FILL ME */
@@ -49,9 +39,9 @@ enum {
   /* FILL ME */
 };
 
-static void gst_videofilter_base_init (GstVideofilterClass *klass);
-static void	gst_videofilter_class_init	(GstVideofilterClass *klass);
-static void	gst_videofilter_init		(GstVideofilter *videofilter);
+static void     gst_videofilter_base_init       (gpointer g_class);
+static void	gst_videofilter_class_init	(gpointer g_class, gpointer class_data);
+static void	gst_videofilter_init		(GTypeInstance *instance, gpointer g_class);
 
 static void	gst_videofilter_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void	gst_videofilter_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
@@ -70,14 +60,14 @@ gst_videofilter_get_type (void)
   if (!videofilter_type) {
     static const GTypeInfo videofilter_info = {
       sizeof(GstVideofilterClass),
-      (GBaseInitFunc)gst_videofilter_base_init,
+      gst_videofilter_base_init,
       NULL,
-      (GClassInitFunc)gst_videofilter_class_init,
+      gst_videofilter_class_init,
       NULL,
       NULL,
       sizeof(GstVideofilter),
       0,
-      (GInstanceInitFunc)gst_videofilter_init,
+      gst_videofilter_init,
     };
     videofilter_type = g_type_register_static(GST_TYPE_ELEMENT,
 	"GstVideofilter", &videofilter_info, G_TYPE_FLAG_ABSTRACT);
@@ -85,16 +75,29 @@ gst_videofilter_get_type (void)
   return videofilter_type;
 }
 
-static void gst_videofilter_base_init (GstVideofilterClass *klass)
+static void gst_videofilter_base_init (gpointer g_class)
 {
+  static GstElementDetails videofilter_details = {
+    "Video scaler",
+    "Filter/Effect/Video",
+    "Resizes video",
+    "David Schleef <ds@schleef.org>"
+  };
+  GstVideofilterClass *klass = (GstVideofilterClass *) g_class;
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
   klass->formats = g_ptr_array_new();
+
+  gst_element_class_set_details (element_class, &videofilter_details);
 }
 
-static void gst_videofilter_class_init (GstVideofilterClass *klass)
+static void gst_videofilter_class_init (gpointer g_class, gpointer class_data)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
+  GstVideofilterClass *klass;
 
+  klass = (GstVideofilterClass *)g_class;
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
@@ -104,44 +107,6 @@ static void gst_videofilter_class_init (GstVideofilterClass *klass)
   gobject_class->get_property = gst_videofilter_get_property;
 }
 
-#if 0
-static GstPadTemplate *
-gst_videofilter_src_template_factory(void)
-{
-  static GstPadTemplate *templ = NULL;
-
-  if(!templ){
-    GstCaps *caps = GST_CAPS_NEW("src","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-
-    caps = gst_caps_intersect(caps, gst_videofilter_get_capslist ());
-
-    templ = GST_PAD_TEMPLATE_NEW("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps);
-  }
-  return templ;
-}
-
-static GstPadTemplate *
-gst_videofilter_sink_template_factory(void)
-{
-  static GstPadTemplate *templ = NULL;
-
-  if(!templ){
-    GstCaps *caps = GST_CAPS_NEW("src","video/x-raw-yuv",
-		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
-		"framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
-
-    caps = gst_caps_intersect(caps, gst_videofilter_get_capslist ());
-
-    templ = GST_PAD_TEMPLATE_NEW("src", GST_PAD_SINK, GST_PAD_ALWAYS, caps);
-  }
-  return templ;
-}
-#endif
-
 static GstCaps *gst_videofilter_format_get_caps(GstVideofilterFormat *format)
 {
   unsigned int fourcc;
@@ -210,7 +175,6 @@ gst_videofilter_sink_getcaps (GstPad *pad, GstCaps *caps)
 
   /* get list of peer's caps */
   peercaps = gst_pad_get_allowed_caps (videofilter->srcpad);
-  //g_print("peercaps: %s\n", gst_caps_to_string(peercaps));
 
   /* FIXME videofilter doesn't allow passthru of video formats it
    * doesn't understand. */
@@ -222,8 +186,6 @@ gst_videofilter_sink_getcaps (GstPad *pad, GstCaps *caps)
     GstCaps *fromcaps = gst_videofilter_format_get_caps(g_ptr_array_index(
 	  klass->formats,i));
 
-    //g_print("testing caps: %s\n", gst_caps_to_string(fromcaps));
-
     icaps = gst_caps_intersect(fromcaps, peercaps);
     //if(gst_caps_is_always_compatible(fromcaps, peercaps)){
     if(icaps != NULL){
@@ -337,34 +299,33 @@ gst_videofilter_sink_link (GstPad *pad, GstCaps *caps)
 }
 
 static void
-gst_videofilter_init (GstVideofilter *videofilter)
+gst_videofilter_init (GTypeInstance *instance, gpointer g_class)
 {
+  GstVideofilter *videofilter = GST_VIDEOFILTER (instance);
+  GstPadTemplate *pad_template;
+
   GST_DEBUG("gst_videofilter_init");
 
-}
-
-void
-gst_videofilter_postinit (GstVideofilter *videofilter)
-{
-  /* sinkpad and srcpad should be inited by subclass */
-
-  /* FIXME */
-  //videofilter->sinkpad = NULL;
+  pad_template = gst_element_class_get_pad_template(GST_ELEMENT_CLASS(g_class),
+      "sink");
+  g_return_if_fail(pad_template != NULL);
+  videofilter->sinkpad = gst_pad_new_from_template(pad_template, "sink");
   gst_element_add_pad(GST_ELEMENT(videofilter),videofilter->sinkpad);
   gst_pad_set_chain_function(videofilter->sinkpad,gst_videofilter_chain);
   gst_pad_set_link_function(videofilter->sinkpad,gst_videofilter_sink_link);
   gst_pad_set_getcaps_function(videofilter->sinkpad,gst_videofilter_sink_getcaps);
 
-  /* FIXME */
-  //videofilter->srcpad = NULL;
+  pad_template = gst_element_class_get_pad_template(GST_ELEMENT_CLASS(g_class),
+      "src");
+  g_return_if_fail(pad_template != NULL);
+  videofilter->srcpad = gst_pad_new_from_template(pad_template, "src");
   gst_element_add_pad(GST_ELEMENT(videofilter),videofilter->srcpad);
   gst_pad_set_link_function(videofilter->srcpad,gst_videofilter_src_link);
-  //gst_pad_set_getcaps_function(videofilter->srcpad,gst_videofilter_getcaps);
+  //gst_pad_set_getcaps_function(videofilter->srcpad,gst_videofilter_src_getcaps);
 
   videofilter->inited = FALSE;
 }
 
-
 static void
 gst_videofilter_chain (GstPad *pad, GstData *_data)
 {
@@ -564,33 +525,42 @@ void gst_videofilter_class_add_format(GstVideofilterClass *videofilterclass,
   g_ptr_array_add(videofilterclass->formats, format);
 }
 
-static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+void gst_videofilter_class_add_pad_templates (GstVideofilterClass *videofilter_class)
 {
-#if 0
-  GstElementFactory *factory;
+  GstCaps *caps;
+  GstElementClass *element_class = GST_ELEMENT_CLASS (videofilter_class);
 
-  /* create an elementfactory for the videofilter element */
-  factory = gst_element_factory_new("videofilter",GST_TYPE_VIDEOFILTER,
-                                   &videofilter_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
+  caps = GST_CAPS_NEW("src","video/x-raw-yuv",
+		"width", GST_PROPS_INT_RANGE (0, G_MAXINT),
+		"height", GST_PROPS_INT_RANGE (0, G_MAXINT),
+		"framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT));
 
-  /* FIXME: we need to init the class before we can call the
-   * pad template factories */
-  gst_element_factory_add_pad_template (factory,
-      GST_PAD_TEMPLATE_GET (gst_videofilter_sink_template_factory));
-  gst_element_factory_add_pad_template (factory,
-      GST_PAD_TEMPLATE_GET (gst_videofilter_src_template_factory));
+  gst_element_class_add_pad_template (element_class,
+      GST_PAD_TEMPLATE_NEW("src", GST_PAD_SRC, GST_PAD_ALWAYS, 
+        gst_caps_intersect(caps,
+          gst_videofilter_class_get_capslist (videofilter_class))));
 
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-#endif
+  gst_element_class_add_pad_template (element_class,
+      GST_PAD_TEMPLATE_NEW("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
+        gst_caps_intersect(caps,
+          gst_videofilter_class_get_capslist (videofilter_class))));
+}
 
+static gboolean
+plugin_init (GstPlugin *plugin)
+{
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "gstvideofilter",
-  plugin_init
-};
+  "Video filter parent class",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
