@@ -21,20 +21,24 @@
 #include <config.h>
 #endif
 
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <unistd.h>
+
 #include "v4l2_calls.h"
 #include "gstv4l2tuner.h"
 #include "gstv4l2xoverlay.h"
 #include "gstv4l2colorbalance.h"
 
+#include <gst/propertyprobe/propertyprobe.h>
+
 /* elementfactory details */
 static GstElementDetails gst_v4l2element_details = {
 	"Generic video4linux2 Element",
 	"Generic/Video",
-	"LGPL",
 	"Generic plugin for handling common video4linux2 calls",
-	VERSION,
-	"Ronald Bultje <rbultje@ronald.bitfreak.net>",
-	"(C) 2002",
+	"Ronald Bultje <rbultje@ronald.bitfreak.net>"
 };
 
 /* V4l2Element signals and args */
@@ -54,6 +58,7 @@ enum {
 
 
 static void	gst_v4l2element_class_init	(GstV4l2ElementClass *klass);
+static void	gst_v4l2element_base_init	(GstV4l2ElementClass *klass);
 static void	gst_v4l2element_init		(GstV4l2Element *v4lelement);
 static void	gst_v4l2element_dispose		(GObject        *object);
 static void	gst_v4l2element_set_property	(GObject        *object,
@@ -101,6 +106,166 @@ gst_v4l2_interface_init (GstInterfaceClass *klass)
 }
 
 
+static const GList *
+gst_v4l2_probe_get_properties (GstPropertyProbe *probe)
+{
+	GObjectClass *klass = G_OBJECT_GET_CLASS (probe);
+	static GList *list = NULL;
+
+	if (!list) {
+		list = g_list_append (NULL,
+				g_object_class_find_property (klass, "device"));
+	}
+
+	return list;
+}
+
+static gboolean
+gst_v4l2_class_probe_devices (GstV4l2ElementClass *klass,
+			      gboolean             check)
+{
+	static gboolean init = FALSE;
+	static GList *devices = NULL;
+
+	if (!init && !check) {
+		gchar *dev_base[] = { "/dev/video", "/dev/v4l/video", NULL };
+		gint base, n, fd;
+
+		while (devices) {
+			GList *item = devices;
+			gchar *device = item->data;
+
+			devices = g_list_remove (devices, item);
+			g_free (device);
+		}
+
+		/* detect /dev entries */
+		for (n = 0; n < 64; n++) {
+			for (base = 0; dev_base[base] != NULL; base++) {
+				struct stat s;
+				gchar *device = g_strdup_printf ("%s%d",
+							dev_base[base], n);
+
+				/* does the /dev/ entry exist at all? */
+				if (stat (device, &s) == 0) {
+					/* yes: is a device attached? */
+					if ((fd = open (device, O_RDONLY)) > 0 ||
+					    errno == EBUSY) {
+						if (fd > 0)
+							close (fd);
+
+						devices =
+							g_list_append (devices,
+								       device);
+						break;
+					}
+				}
+				g_free (device);
+			}
+		}
+
+		init = TRUE;
+	}
+
+	klass->devices = devices;
+
+	return init;
+}
+
+static void
+gst_v4l2_probe_probe_property (GstPropertyProbe *probe,
+			       guint             prop_id,
+			       const GParamSpec *pspec)
+{
+	GstV4l2ElementClass *klass = GST_V4L2ELEMENT_GET_CLASS (probe);
+
+	switch (prop_id) {
+		case ARG_DEVICE:
+			gst_v4l2_class_probe_devices (klass, FALSE);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+			break;
+	}
+}
+
+static gboolean
+gst_v4l2_probe_needs_probe (GstPropertyProbe *probe,
+			    guint             prop_id,
+			    const GParamSpec *pspec)
+{
+	GstV4l2ElementClass *klass = GST_V4L2ELEMENT_GET_CLASS (probe);
+	gboolean ret = FALSE;
+
+	switch (prop_id) {
+		case ARG_DEVICE:
+			ret = !gst_v4l2_class_probe_devices (klass, TRUE);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+			break;
+	}
+
+	return ret;
+}
+
+static GValueArray *
+gst_v4l2_class_list_devices (GstV4l2ElementClass *klass)
+{
+	GValueArray *array;
+	GValue value = { 0 };
+	GList *item;
+
+	if (!klass->devices)
+		return NULL;
+
+	array = g_value_array_new (g_list_length (klass->devices));
+	item = klass->devices;
+	g_value_init (&value, G_TYPE_STRING);
+	while (item) {
+		gchar *device = item->data;
+
+		g_value_set_string (&value, device);
+		g_value_array_append (array, &value);
+
+		item = item->next;
+	}
+	g_value_unset (&value);
+
+	return array;
+}
+
+static GValueArray *
+gst_v4l2_probe_get_values (GstPropertyProbe *probe,
+			   guint             prop_id,
+			   const GParamSpec *pspec)
+{
+	GstV4l2ElementClass *klass = GST_V4L2ELEMENT_GET_CLASS (probe);
+	GValueArray *array = NULL;
+
+	switch (prop_id) {
+		case ARG_DEVICE:
+			array = gst_v4l2_class_list_devices (klass);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+			break;
+	}
+
+	return array;
+}
+
+
+static void
+gst_v4l2_property_probe_interface_init (GstPropertyProbeInterface *iface)
+{
+	iface->get_properties = gst_v4l2_probe_get_properties;
+	iface->probe_property = gst_v4l2_probe_probe_property;
+	iface->needs_probe    = gst_v4l2_probe_needs_probe;
+	iface->get_values     = gst_v4l2_probe_get_values;
+}
+
+
 GType
 gst_v4l2element_get_type (void)
 {
@@ -109,7 +274,7 @@ gst_v4l2element_get_type (void)
 	if (!v4l2element_type) {
 		static const GTypeInfo v4l2element_info = {
 			sizeof(GstV4l2ElementClass),
-			NULL,
+			(GBaseInitFunc) gst_v4l2element_base_init,
 			NULL,
 			(GClassInitFunc) gst_v4l2element_class_init,
 			NULL,
@@ -139,6 +304,11 @@ gst_v4l2element_get_type (void)
 			NULL,
 			NULL,
 		};
+		static const GInterfaceInfo v4l2_propertyprobe_info = {
+			(GInterfaceInitFunc) gst_v4l2_property_probe_interface_init,
+			NULL,
+			NULL,
+		};
 
 		v4l2element_type =
 			g_type_register_static(GST_TYPE_ELEMENT,
@@ -156,6 +326,9 @@ gst_v4l2element_get_type (void)
 		g_type_add_interface_static (v4l2element_type,
 					     GST_TYPE_COLOR_BALANCE,
 					     &v4l2_colorbalance_info);
+		g_type_add_interface_static (v4l2element_type,
+					     GST_TYPE_PROPERTY_PROBE,
+					     &v4l2_propertyprobe_info);
 	}
 
 	return v4l2element_type;
@@ -191,6 +364,16 @@ gst_v4l2_device_get_type (void)
 	return v4l2_device_type;
 }
 
+static void
+gst_v4l2element_base_init (GstV4l2ElementClass *klass)
+{
+	GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
+
+	klass->devices = NULL;
+
+	gst_element_class_set_details (gstelement_class,
+				       &gst_v4l2element_details);
+}
 
 static void
 gst_v4l2element_class_init (GstV4l2ElementClass *klass)
@@ -381,22 +564,3 @@ gst_v4l2element_change_state (GstElement *element)
 
 	return GST_STATE_SUCCESS;
 }
-
-
-gboolean
-gst_v4l2element_factory_init (GstPlugin *plugin)
-{
-	GstElementFactory *factory;
-
-	/* we can run without... But not yet. ;). */
-	if (!gst_library_load ("xwindowlistener"))
-		return FALSE;
-
-	/* create an elementfactory for the v4l2element */
-	factory = gst_element_factory_new("v4l2element", GST_TYPE_V4L2ELEMENT,
-				&gst_v4l2element_details);
-	g_return_val_if_fail(factory != NULL, FALSE);
-	gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-	return TRUE;
-}
