@@ -27,34 +27,41 @@
 
 /*#define GST_DEBUG_FORCE_DISABLE*/
 
+#include <gst/navigation/navigation.h>
+#include <gst/xoverlay/xoverlay.h>
+
 #include "xvideosink.h"
 
 /* elementfactory information */
 static GstElementDetails gst_xvideosink_details = {
   "Video sink",
   "Sink/Video",
-  "LGPL",
   "A general X Window video sink",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2000",
+  "Wim Taymans <wim.taymans@chello.be>"
 };
 
 /* xvideosink signals and args */
 
 enum {
   ARG_0,
-  ARG_SCALE,
   ARG_DISABLE_XV,
-  ARG_TOPLEVEL,
   ARG_AUTOSIZE,
-  ARG_NEED_NEW_WINDOW,
 };
 
+static void		gst_xvideosink_base_init	(gpointer g_class);
 static void		gst_xvideosink_class_init	(GstXVideoSinkClass *klass);
 static void		gst_xvideosink_init		(GstXVideoSink *xvideosink);
 static void 		gst_xvideosink_dispose 		(GObject *object);
 
+static void		gst_xvideosink_interface_init	(GstInterfaceClass *klass);
+static gboolean		gst_xvideosink_interface_supported (GstInterface *iface, GType type);
+
+static void		gst_xvideosink_xoverlay_init	(GstXOverlayClass *klass);
+static void		gst_xvideosink_xoverlay_set_xwindow_id (GstXOverlay *overlay, XID xwindow_id);
+
+static void             gst_xvideosink_navigation_init  (GstNavigationInterface *iface);
+static void             gst_xvideosink_navigation_send_event (GstNavigation *navigation, GstStructure *structure);
+
 static void		gst_xvideosink_chain		(GstPad *pad, GstData *_data);
 static GstElementStateReturn
 			gst_xvideosink_change_state 	(GstElement *element);
@@ -99,7 +106,7 @@ gst_xvideosink_get_type (void)
   if (!xvideosink_type) {
     static const GTypeInfo xvideosink_info = {
       sizeof (GstXVideoSinkClass),
-      NULL,
+      gst_xvideosink_base_init,
       NULL,
       (GClassInitFunc) gst_xvideosink_class_init,
       NULL,
@@ -108,13 +115,44 @@ gst_xvideosink_get_type (void)
       0,
       (GInstanceInitFunc) gst_xvideosink_init,
     };
+    static const GInterfaceInfo iface_info = {
+      (GInterfaceInitFunc) gst_xvideosink_interface_init,
+      NULL,
+      NULL,
+    };
+    static const GInterfaceInfo navigation_info = {
+      (GInterfaceInitFunc) gst_xvideosink_navigation_init,
+      NULL,
+      NULL,
+    };
+    static const GInterfaceInfo xoverlay_info = {
+      (GInterfaceInitFunc) gst_xvideosink_xoverlay_init,
+      NULL,
+      NULL,
+    };
     xvideosink_type = g_type_register_static(GST_TYPE_VIDEOSINK,
                                              "GstXVideoSink",
                                              &xvideosink_info, 0);
+    g_type_add_interface_static (xvideosink_type, GST_TYPE_INTERFACE,
+	&iface_info);
+    g_type_add_interface_static (xvideosink_type, GST_TYPE_NAVIGATION,
+        &navigation_info);
+    g_type_add_interface_static (xvideosink_type, GST_TYPE_X_OVERLAY,
+	&xoverlay_info);
   }
   return xvideosink_type;
 }
 
+static void
+gst_xvideosink_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_set_details (element_class, &gst_xvideosink_details);
+
+  gst_element_class_add_pad_template (element_class, 
+    GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory));
+}
 static void
 gst_xvideosink_class_init (GstXVideoSinkClass *klass)
 {
@@ -131,17 +169,9 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DISABLE_XV,
     g_param_spec_boolean ("disable_xv", "Disable XV", "Disable Xv images",
                           TRUE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_TOPLEVEL,
-    g_param_spec_boolean ("toplevel", "Toplevel", "Create a toplevel window",
-                          TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_AUTOSIZE,
     g_param_spec_boolean ("auto_size", "Auto Size", "Resizes the window to negotiated size",
                           TRUE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NEED_NEW_WINDOW,
-    g_param_spec_boolean ("need_new_window", 
-	                  "Needs a new X window", 
-			  "Request that a new X window be created for embedding",
-                          TRUE, G_PARAM_WRITABLE)); 
 
   gobject_class->set_property = gst_xvideosink_set_property;
   gobject_class->get_property = gst_xvideosink_get_property;
@@ -156,6 +186,71 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
   gstvs_class->set_geometry = gst_xvideosink_set_geometry;*/
 }
 
+static void
+gst_xvideosink_interface_init (GstInterfaceClass *klass)
+{
+  klass->supported = gst_xvideosink_interface_supported;
+}
+
+static gboolean
+gst_xvideosink_interface_supported (GstInterface *iface, GType type)
+{
+  g_assert (type == GST_TYPE_NAVIGATION ||
+	    type == GST_TYPE_X_OVERLAY);
+
+  return (GST_STATE (iface) != GST_STATE_NULL);
+}
+
+static void
+gst_xvideosink_navigation_init (GstNavigationInterface *iface)
+{
+  iface->send_event = gst_xvideosink_navigation_send_event;
+}
+
+static void
+gst_xvideosink_navigation_send_event (GstNavigation *navigation,
+    GstStructure *structure)
+{
+  GstXVideoSink *xvideosink = GST_XVIDEOSINK (navigation);
+  GstEvent *event;
+
+  event = gst_event_new (GST_EVENT_NAVIGATION);
+  /*GST_EVENT_TIMESTAMP (event) = 0;*/
+  event->event_data.structure.structure = structure;
+
+  /* FIXME 
+   * Obviously, the pointer x,y coordinates need to be adjusted by the
+   * window size and relation to the bounding window. */
+
+  gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD(xvideosink)),
+      event);
+}
+
+static void
+gst_xvideosink_xoverlay_init (GstXOverlayClass *klass)
+{
+  klass->set_xwindow_id = gst_xvideosink_xoverlay_set_xwindow_id;
+}
+
+static void
+gst_xvideosink_xoverlay_set_xwindow_id (GstXOverlay *overlay, XID xwindow_id)
+{
+  GstXVideoSink *xvideosink = GST_XVIDEOSINK (overlay);
+
+  if (xwindow_id == xvideosink->xwindow_id)
+    return;
+
+  xvideosink->xwindow_id = xwindow_id;
+
+  /* if we already had one - recreate it */
+  if (xvideosink->window) {
+    g_mutex_lock (xvideosink->lock);
+    gst_xvideosink_xwindow_free (xvideosink);
+    gst_xvideosink_xwindow_new (xvideosink);
+    g_mutex_unlock (xvideosink->lock);
+  }
+}
+
 static void
 gst_xvideosink_imagepool_clear (GstXVideoSink *xvideosink)
 {
@@ -190,10 +285,16 @@ gst_xvideosink_buffer_new (GstBufferPool *pool,
   
   xvideosink = GST_XVIDEOSINK (user_data);
 
-  g_mutex_lock (xvideosink->pool_lock);
-  if (!xvideosink->image_pool) {
-    g_mutex_unlock (xvideosink->pool_lock);
+  image = NULL;
 
+  g_mutex_lock (xvideosink->pool_lock);
+  if (xvideosink->image_pool) {
+    image = xvideosink->image_pool->data;
+    xvideosink->image_pool = g_slist_delete_link (xvideosink->image_pool, xvideosink->image_pool);
+  }
+  g_mutex_unlock (xvideosink->pool_lock);
+
+  if (image == NULL) {
     /* we have to lock the X connection */
     g_mutex_lock (xvideosink->lock);
     if (xvideosink->image_format->type == GST_TYPE_XIMAGE) {
@@ -214,16 +315,13 @@ gst_xvideosink_buffer_new (GstBufferPool *pool,
     }
 #endif
     else {
+      g_mutex_unlock (xvideosink->lock);
       g_assert_not_reached();
       return NULL;
     }
     g_mutex_unlock (xvideosink->lock);
   }
-  else {
-    image = xvideosink->image_pool->data;
-    xvideosink->image_pool = g_slist_delete_link (xvideosink->image_pool, xvideosink->image_pool);
-    g_mutex_unlock (xvideosink->pool_lock);
-  }
+
   if (image == NULL) {
     gst_element_error(GST_ELEMENT (xvideosink), "image creation failed");
     /* FIXME: need better error handling here */
@@ -267,7 +365,7 @@ gst_xvideosink_buffer_free (GstBufferPool *pool, GstBuffer *buffer, gpointer use
         image_window == GST_IMAGE_WINDOW (xvideosink->image)) 
     { 
       keep_buffer = TRUE;
-    } 
+    }
   }
   g_mutex_unlock(xvideosink->lock);
 
@@ -316,7 +414,8 @@ gst_xvideosink_get_real_size (GstXVideoSink *xvideosink,
 {
   gint pwidth, pheight;
 
-  gst_video_sink_get_geometry (GST_VIDEOSINK (xvideosink), real_x, real_y);
+  *real_x = GST_VIDEOSINK_WIDTH (xvideosink);
+  *real_y = GST_VIDEOSINK_HEIGHT (xvideosink);
   pwidth  = xvideosink->pixel_width;
   pheight = xvideosink->pixel_height;
 
@@ -476,14 +575,12 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
 
   /* one-time initialization */
   xvideosink->window = NULL;
-  xvideosink->toplevel = TRUE;
+  xvideosink->xwindow_id = 0;
   xvideosink->image_format = NULL;
   xvideosink->lock = g_mutex_new ();
 
   xvideosink->image_pool = NULL;
   xvideosink->pool_lock = g_mutex_new ();
-  xvideosink->send_xid = FALSE;
-  xvideosink->need_new_window = FALSE;
 
   xvideosink->image = NULL;
 
@@ -595,7 +692,6 @@ gst_xvideosink_chain (GstPad *pad, GstData *_data)
     }
   }
 
-  gst_video_sink_frame_displayed (GST_VIDEOSINK (xvideosink));
   g_mutex_unlock (xvideosink->lock);
 
   if (GST_VIDEOSINK_CLOCK (xvideosink)) {
@@ -605,13 +701,6 @@ gst_xvideosink_chain (GstPad *pad, GstData *_data)
     xvideosink->correction = 0;
   }
 
-  if (xvideosink->send_xid){
-    gst_video_sink_got_video_out (
-      GST_VIDEOSINK (xvideosink),
-      (gpointer) GST_XWINDOW_XID (xvideosink->window));
-    xvideosink->send_xid = FALSE;
-  }
-
   gst_buffer_unref (buf);
 }
 
@@ -631,24 +720,9 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
     case ARG_DISABLE_XV:
       xvideosink->disable_xv = g_value_get_boolean (value);
       break;
-    case ARG_TOPLEVEL:
-      xvideosink->toplevel = g_value_get_boolean (value);
-      break;
     case ARG_AUTOSIZE:
       xvideosink->auto_size = g_value_get_boolean (value);
       break;
-    case ARG_NEED_NEW_WINDOW:
-      if (g_value_get_boolean (value)) {
-        if (xvideosink->window) {
-          g_mutex_lock (xvideosink->lock);
-          gst_xvideosink_xwindow_free (xvideosink);
-          gst_xvideosink_xwindow_new (xvideosink);
-          g_mutex_unlock (xvideosink->lock);
-	} else {
-	  xvideosink->need_new_window = TRUE;
-	}
-      }
-      break;
     default:
       break;
   }
@@ -666,9 +740,6 @@ gst_xvideosink_get_property (GObject *object, guint prop_id, GValue *value, GPar
     case ARG_DISABLE_XV:
       g_value_set_boolean (value, xvideosink->disable_xv);
       break;
-    case ARG_TOPLEVEL:
-      g_value_set_boolean (value, xvideosink->toplevel);
-      break;
     case ARG_AUTOSIZE:
       g_value_set_boolean (value, xvideosink->auto_size);
       break;
@@ -730,7 +801,7 @@ gst_xvideosink_xwindow_new (GstXVideoSink *xvideosink)
 {
   xvideosink->window = _gst_xwindow_new (GST_VIDEOSINK_WIDTH (xvideosink), 
 	                                 GST_VIDEOSINK_HEIGHT (xvideosink), 
-					 xvideosink->toplevel);
+					 xvideosink->xwindow_id);
   if (!xvideosink->window) {
     gst_element_error (GST_ELEMENT (xvideosink), "could not create X window");
     return FALSE;
@@ -740,9 +811,6 @@ gst_xvideosink_xwindow_new (GstXVideoSink *xvideosink)
                                  GST_VIDEOSINK_WIDTH (xvideosink),
                                  GST_VIDEOSINK_HEIGHT (xvideosink));
   
-  xvideosink->send_xid = TRUE;
-  xvideosink->need_new_window = FALSE;
-
   if (!xvideosink->image_format) {
     return TRUE;
   } else {
@@ -807,31 +875,27 @@ gst_xvideosink_image_free (GstXVideoSink *xvideosink)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
   /* Loading the library containing GstVideoSink, our parent object */
   if (!gst_library_load ("gstvideo"))
     return FALSE;
   
-  /* create an elementfactory for the xvideosink element */
-  factory = gst_element_factory_new ("xvideosink", GST_TYPE_XVIDEOSINK,
-                                     &gst_xvideosink_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (
-    factory, 
-    GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "xvideosink", GST_RANK_NONE, GST_TYPE_XVIDEOSINK))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "xvideosink",
-  plugin_init
-};
+  "A general X Window video sink",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_COPYRIGHT,
+  GST_PACKAGE,
+  GST_ORIGIN
+);
