@@ -46,7 +46,7 @@ main (int argc, char *argv[])
 				  NULL));
 
   if (!gst_element_link_filtered (volume, fakesink, caps))
-    g_error ("Could not link_filtered volume and fakesink");
+    g_error ("Could not connect_filtered volume and fakesink");
 
   /* force samplerate on sinesrc ! spider */
   caps = gst_caps_new ("samplerate", "audio/raw",
@@ -54,7 +54,7 @@ main (int argc, char *argv[])
 				  "rate", GST_PROPS_INT (44100),
 				  NULL));
   if (!gst_element_link_filtered (sinesrc, spider, caps))
-    g_error ("Could not link_filtered sinesrc and spider");
+    g_error ("Could not connect_filtered sinesrc and spider");
 
   gst_element_link (spider, volume);
   g_print ("Setting pipeline to ready\n");
